#!/usr/bin/env python

""" Parse the doxygen xml files to create cross-ref files that will be read by
doxygen to build web pages. These currently link examples to classes
and methods to classes.
"""

import xml.etree.ElementTree as ET
import tools
import os.path
import sys

creates = {}
examples_classes = {}
examples_functions = {}
takes = {}

verbose = False


def get_class_name(et):
    name = et.attrib['refid']
    nname = name[5:].replace("_1", ":").replace("__", "_")
    if not nname.startswith("IMP"):
        return None
    return nname


def _cleanup_name(n):
    if n.find("href") != -1:
        return n
    if n.find("::") != -1:
        sp = n.split("::")
        if len(sp) == 5:
            return None
        return "::".join(sp)
    if n == 'IMP_NEW()':
        # Workaround for (unqualified) macro
        return "IMP::" + n
    if n.find(".py") != -1 or n.find(".cpp") != -1:
        rep = n.replace("/", "_2").replace(".", "_8")
        return ("[%s](%s)" % (n, rep + "-example.html"))
    else:
        # fix later
        return None


def get_function_name(et):
    name = et.text
    nname = name.replace(".", "::")
    if not nname.startswith("IMP"):
        return None
    return nname


def get_function_link(name, et, mname):
    nicename = name + "::" + et.find(".//name").text
    # Skip template specializations
    if '<' in nicename:
        return None
    refid = et.attrib['id']
    split = refid.split("_1")
    fname = "_1".join(split[:-1]) + ".html"
    tag = split[-1]
    return "<a href=\"" + fname + "#" + tag + "\">" + nicename + "()</a>"


def _add_to_list(table, key, value):
    if key in table:
        if value not in table[key]:
            table[key].append(value)
    else:
        table[key] = [value]


def _add_example_class_ref(example_name, class_name):
    if not class_name:
        return
    _add_to_list(examples_classes, class_name, example_name)


def _add_example_function_ref(example_name, function_name):
    if not function_name:
        return
    _add_to_list(examples_functions, function_name + "()", example_name)


def _add_takes_ref(name, class_name):
    if not class_name:
        return
    _add_to_list(takes, class_name, name)


def _add_creates_ref(name, class_name):
    if not class_name:
        return
    _add_to_list(creates, class_name, name)


def get_example_name(et):
    nm = et.find('.//location').attrib['file']
    st = os.path.join("build", "doc", "examples")
    rnm = nm[nm.find(st) + len(st) + 1:]
    return rnm


def get_example_2_name(et):
    nm = et.find('.//compoundname').text
    return nm


def traverse_example(name, et):
    if et.tag == 'ref':
        if et.attrib['kindref'] == "compound":
            _add_example_class_ref(name, get_class_name(et))
        if et.attrib['kindref'] == "member":
            _add_example_function_ref(name, get_function_name(et))
    for child in et:
        traverse_example(name, child)


def traverse_example_2(name, et):
    if et.tag == 'ref':
        if et.attrib['kindref'] == "compound":
            _add_example_class_ref(name, get_class_name(et))
        if et.attrib['kindref'] == "member":
            _add_example_function_ref(name, get_function_name(et))
    for child in et:
        traverse_example(name, child)


def get_file_class_name(et):
    nm = et.find('.//compoundname').text
    return nm


def traverse_ret(name, et):
    if et.tag == 'ref':
        if et.attrib['kindref'] == "compound":
            _add_creates_ref(name, get_class_name(et))
    for child in et:
        traverse_ret(name, child)


def traverse_param(name, et):
    if et.tag == 'ref':
        if et.attrib['kindref'] == "compound":
            _add_takes_ref(name, get_class_name(et))
    for child in et:
        traverse_param(name, child)


def traverse_class(name, et, module):
    if et.tag in ['listofallmembers', 'collaborationgraph',
                  'inheritancegraph']:
        return
    if et.tag == 'memberdef' and et.attrib["kind"] == "function":
        membername = get_function_link(name, et, module)
        if membername:
            traverse_ret(membername, et.find(".//type"))
            for p in et.findall(".//param"):
                traverse_param(membername, p)
    else:
        for child in et:
            traverse_class(name, child, module)


def get_namespace_name(et):
    return et.find(".//compoundname").text


def traverse_namespace(name, et, module):
    if et.tag == 'memberdef' and et.attrib["kind"] == "function":
        membername = get_function_link(name, et, module)
        if membername:
            traverse_ret(membername, et.find(".//type"))
            for p in et.findall(".//param"):
                traverse_param(membername, p)
    else:
        for child in et:
            traverse_namespace(name, child, module)


def create_index(
    title,
    ref,
    other_indexes,
    description,
    links,
    target,
    key_name,
        target_name):
    out = open(target, "w")
    out.write("%s {#%s}\n%s\n\n" % (title, ref, "=" * len(title)))
    out.write("# Overview #\n")
    out.write("[TOC]\n")
    out.write(description + "\n\n")
    out.write(
        "See also " + ", ".join(["[%s](@ref %s)" %
                                 (x[0], x[1]) for x in other_indexes]) + "\n")
    keys = sorted(links.keys())
    keys_by_module = {}
    for k in keys:
        kc = _cleanup_name(k)
        if not kc:
            continue
        split_kc = kc.split("::")
        if len(split_kc) == 2:  # kernel
            m = 'kernel'
        else:
            m = split_kc[1]
        if m not in keys_by_module.keys():
            keys_by_module[m] = []
        keys_by_module[m].append(k)
    modules = sorted(keys_by_module.keys())
    # Put kernel first
    modules.remove('kernel')
    modules.insert(0, 'kernel')
    for m in modules:
        if m == 'kernel':
            out.write("# IMP (%s) # {#%s_%s}\n" % (m, ref, m))
        else:
            out.write("# IMP.%s # {#%s_%s}\n" % (m, ref, m))
        out.write("<table><tr>\n")
        out.write("<th>%s</th><th>%s</th></tr>\n" % (key_name, target_name))
        for k in keys_by_module[m]:

            cn = _cleanup_name(k)
            if not cn:
                continue
            out.write("<tr><td>@ref %s</td>" % cn)
            # suppress same names as they aren't very useful
            seen = []
            entry = []
            for ln in links[k]:
                cn = _cleanup_name(ln)
                if not cn:
                    continue
                if cn and cn not in seen:
                    entry.append(cn)
                seen.append(cn)
            out.write("<td>%s</td></tr>\n" % ", ".join(entry))
        out.write("</table>\n")


def main():
    # glob.glob(os.path.join("build", "doxygen", "xml", "*.xml")):
    if len(sys.argv) > 1:
        files = sys.argv[1:]
    else:
        files = tools.get_glob([os.path.join("doxygen", "ref", "xml",
                                             "*.xml")])
    for f in files:
        module = os.path.split(os.path.split(os.path.split(f)[0])[0])[1]
        try:
            et = ET.parse(f)
        except ET.ParseError:
            sys.stderr.write("ERROR parsing", f, "\n")
        fname = os.path.basename(f)
        if fname.startswith("namespaceIMP"):
            if verbose:
                print("namespace", fname)
            traverse_namespace(
                get_namespace_name(et.getroot()),
                et.getroot(),
                module)
        elif fname.endswith("example.xml"):
            if verbose:
                print("example 2", fname)
            traverse_example_2(get_example_2_name(et.getroot()), et.getroot())
        # skip structs for nwo
        elif fname.startswith("classIMP"):
            if verbose:
                print("class", fname)
            traverse_class(
                get_file_class_name(et.getroot()),
                et.getroot(),
                module)
        else:
            if verbose:
                print("skipping", fname)
    indexes = [("Factory Index", "factory_index"),
               ("Argument Index", "argument_index"),
               ("Class Examples", "class_example_index"),
               ("Function Examples", "function_example_index")]
    create_index(indexes[0][0], indexes[0][1], indexes[1:],
                 "Functions that create objects of a given type:",
                 creates, "doxygen/generated/factory_index.md", "Class",
                 "Factories")
    create_index(indexes[1][0], indexes[1][1], indexes,
                 "Functions that take objects of a given type as arguments:",
                 takes, "doxygen/generated/argument_index.md", "Class",
                 "Users")
    create_index(indexes[2][0], indexes[2][1], indexes,
                 "Examples that use a given class:",
                 examples_classes, "doxygen/generated/class_example_index.md",
                 "Class", "Examples")
    create_index(indexes[3][0], indexes[3][1], indexes[:-1],
                 "Examples that use a given function:",
                 examples_functions,
                 "doxygen/generated/function_example_index.md",
                 "Function", "Examples")


if __name__ == '__main__':
    main()



# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

# This wrapper is part of IMP,
# Copyright 2007-2021 IMP Inventors. All rights reserved.

from __future__ import print_function, division, absolute_import
from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

import _IMP_display

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

class IMP_DISPLAY_SwigPyIterator(object):
    r"""Proxy of C++ swig::IMP_DISPLAY_SwigPyIterator class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    __swig_destroy__ = _IMP_display.delete_IMP_DISPLAY_SwigPyIterator

    def value(self):
        r"""value(IMP_DISPLAY_SwigPyIterator self) -> PyObject *"""
        return _IMP_display.IMP_DISPLAY_SwigPyIterator_value(self)

    def incr(self, n=1):
        r"""incr(IMP_DISPLAY_SwigPyIterator self, size_t n=1) -> IMP_DISPLAY_SwigPyIterator"""
        return _IMP_display.IMP_DISPLAY_SwigPyIterator_incr(self, n)

    def decr(self, n=1):
        r"""decr(IMP_DISPLAY_SwigPyIterator self, size_t n=1) -> IMP_DISPLAY_SwigPyIterator"""
        return _IMP_display.IMP_DISPLAY_SwigPyIterator_decr(self, n)

    def distance(self, x):
        r"""distance(IMP_DISPLAY_SwigPyIterator self, IMP_DISPLAY_SwigPyIterator x) -> ptrdiff_t"""
        return _IMP_display.IMP_DISPLAY_SwigPyIterator_distance(self, x)

    def equal(self, x):
        r"""equal(IMP_DISPLAY_SwigPyIterator self, IMP_DISPLAY_SwigPyIterator x) -> bool"""
        return _IMP_display.IMP_DISPLAY_SwigPyIterator_equal(self, x)

    def copy(self):
        r"""copy(IMP_DISPLAY_SwigPyIterator self) -> IMP_DISPLAY_SwigPyIterator"""
        return _IMP_display.IMP_DISPLAY_SwigPyIterator_copy(self)

    def next(self):
        r"""next(IMP_DISPLAY_SwigPyIterator self) -> PyObject *"""
        return _IMP_display.IMP_DISPLAY_SwigPyIterator_next(self)

    def __next__(self):
        r"""__next__(IMP_DISPLAY_SwigPyIterator self) -> PyObject *"""
        return _IMP_display.IMP_DISPLAY_SwigPyIterator___next__(self)

    def previous(self):
        r"""previous(IMP_DISPLAY_SwigPyIterator self) -> PyObject *"""
        return _IMP_display.IMP_DISPLAY_SwigPyIterator_previous(self)

    def advance(self, n):
        r"""advance(IMP_DISPLAY_SwigPyIterator self, ptrdiff_t n) -> IMP_DISPLAY_SwigPyIterator"""
        return _IMP_display.IMP_DISPLAY_SwigPyIterator_advance(self, n)

    def __eq__(self, x):
        r"""__eq__(IMP_DISPLAY_SwigPyIterator self, IMP_DISPLAY_SwigPyIterator x) -> bool"""
        return _IMP_display.IMP_DISPLAY_SwigPyIterator___eq__(self, x)

    def __ne__(self, x):
        r"""__ne__(IMP_DISPLAY_SwigPyIterator self, IMP_DISPLAY_SwigPyIterator x) -> bool"""
        return _IMP_display.IMP_DISPLAY_SwigPyIterator___ne__(self, x)

    def __iadd__(self, n):
        r"""__iadd__(IMP_DISPLAY_SwigPyIterator self, ptrdiff_t n) -> IMP_DISPLAY_SwigPyIterator"""
        return _IMP_display.IMP_DISPLAY_SwigPyIterator___iadd__(self, n)

    def __isub__(self, n):
        r"""__isub__(IMP_DISPLAY_SwigPyIterator self, ptrdiff_t n) -> IMP_DISPLAY_SwigPyIterator"""
        return _IMP_display.IMP_DISPLAY_SwigPyIterator___isub__(self, n)

    def __add__(self, n):
        r"""__add__(IMP_DISPLAY_SwigPyIterator self, ptrdiff_t n) -> IMP_DISPLAY_SwigPyIterator"""
        return _IMP_display.IMP_DISPLAY_SwigPyIterator___add__(self, n)

    def __sub__(self, *args):
        r"""
        __sub__(IMP_DISPLAY_SwigPyIterator self, ptrdiff_t n) -> IMP_DISPLAY_SwigPyIterator
        __sub__(IMP_DISPLAY_SwigPyIterator self, IMP_DISPLAY_SwigPyIterator x) -> ptrdiff_t
        """
        return _IMP_display.IMP_DISPLAY_SwigPyIterator___sub__(self, *args)
    def __iter__(self):
        return self

# Register IMP_DISPLAY_SwigPyIterator in _IMP_display:
_IMP_display.IMP_DISPLAY_SwigPyIterator_swigregister(IMP_DISPLAY_SwigPyIterator)


_value_types=[]
_object_types=[]
_raii_types=[]
_plural_types=[]

IMP_DEBUG = _IMP_display.IMP_DEBUG

IMP_RELEASE = _IMP_display.IMP_RELEASE

IMP_SILENT = _IMP_display.IMP_SILENT

IMP_PROGRESS = _IMP_display.IMP_PROGRESS

IMP_TERSE = _IMP_display.IMP_TERSE

IMP_VERBOSE = _IMP_display.IMP_VERBOSE

IMP_MEMORY = _IMP_display.IMP_MEMORY

IMP_NONE = _IMP_display.IMP_NONE

IMP_USAGE = _IMP_display.IMP_USAGE

IMP_INTERNAL = _IMP_display.IMP_INTERNAL

IMP_KERNEL_HAS_LOG4CXX = _IMP_display.IMP_KERNEL_HAS_LOG4CXX

IMP_COMPILER_HAS_AUTO = _IMP_display.IMP_COMPILER_HAS_AUTO

IMP_COMPILER_HAS_DEBUG_VECTOR = _IMP_display.IMP_COMPILER_HAS_DEBUG_VECTOR

IMP_COMPILER_HAS_UNIQUE_PTR = _IMP_display.IMP_COMPILER_HAS_UNIQUE_PTR

IMP_KERNEL_HAS_BOOST_RANDOM = _IMP_display.IMP_KERNEL_HAS_BOOST_RANDOM

IMP_KERNEL_HAS_NUMPY = _IMP_display.IMP_KERNEL_HAS_NUMPY

IMP_KERNEL_HAS_GPERFTOOLS = _IMP_display.IMP_KERNEL_HAS_GPERFTOOLS

IMP_KERNEL_HAS_TCMALLOC_HEAPCHECKER = _IMP_display.IMP_KERNEL_HAS_TCMALLOC_HEAPCHECKER

IMP_KERNEL_HAS_TCMALLOC_HEAPPROFILER = _IMP_display.IMP_KERNEL_HAS_TCMALLOC_HEAPPROFILER

IMPKERNEL_SHOW_WARNINGS = _IMP_display.IMPKERNEL_SHOW_WARNINGS


import sys
class _DirectorObjects(object):
    """@internal Simple class to keep references to director objects
       to prevent premature deletion."""
    def __init__(self):
        self._objects = []
    def register(self, obj):
        """Take a reference to a director object; will only work for
           refcounted C++ classes"""
        if hasattr(obj, 'get_ref_count'):
            self._objects.append(obj)
    def cleanup(self):
        """Only drop our reference and allow cleanup by Python if no other
           Python references exist (we hold 3 references: one in self._objects,
           one in x, and one in the argument list for getrefcount) *and* no
           other C++ references exist (the Python object always holds one)"""
        objs = [x for x in self._objects if sys.getrefcount(x) > 3 \
                                         or x.get_ref_count() > 1]
# Do in two steps so the references are kept until the end of the
# function (deleting references may trigger a fresh call to this method)
        self._objects = objs
    def get_object_count(self):
        """Get number of director objects (useful for testing only)"""
        return len(self._objects)
_director_objects = _DirectorObjects()

class _ostream(object):
    r"""Proxy of C++ std::ostream class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    def write(self, osa_buf):
        r"""write(_ostream self, char const * osa_buf)"""
        return _IMP_display._ostream_write(self, osa_buf)

# Register _ostream in _IMP_display:
_IMP_display._ostream_swigregister(_ostream)

IMP_COMPILER_HAS_OVERRIDE = _IMP_display.IMP_COMPILER_HAS_OVERRIDE

IMP_COMPILER_HAS_FINAL = _IMP_display.IMP_COMPILER_HAS_FINAL

IMP_HAS_NOEXCEPT = _IMP_display.IMP_HAS_NOEXCEPT

IMP_C_OPEN_BINARY = _IMP_display.IMP_C_OPEN_BINARY

import IMP
IMP_CGAL_HAS_BOOST_FILESYSTEM = _IMP_display.IMP_CGAL_HAS_BOOST_FILESYSTEM

IMP_CGAL_HAS_BOOST_PROGRAMOPTIONS = _IMP_display.IMP_CGAL_HAS_BOOST_PROGRAMOPTIONS

IMP_CGAL_HAS_BOOST_RANDOM = _IMP_display.IMP_CGAL_HAS_BOOST_RANDOM

IMP_CGAL_HAS_BOOST_SYSTEM = _IMP_display.IMP_CGAL_HAS_BOOST_SYSTEM

IMP_CGAL_HAS_NUMPY = _IMP_display.IMP_CGAL_HAS_NUMPY

IMPCGAL_SHOW_WARNINGS = _IMP_display.IMPCGAL_SHOW_WARNINGS

import IMP.cgal
IMP_ALGEBRA_HAS_IMP_CGAL = _IMP_display.IMP_ALGEBRA_HAS_IMP_CGAL

IMP_ALGEBRA_HAS_BOOST_FILESYSTEM = _IMP_display.IMP_ALGEBRA_HAS_BOOST_FILESYSTEM

IMP_ALGEBRA_HAS_BOOST_PROGRAMOPTIONS = _IMP_display.IMP_ALGEBRA_HAS_BOOST_PROGRAMOPTIONS

IMP_ALGEBRA_HAS_BOOST_RANDOM = _IMP_display.IMP_ALGEBRA_HAS_BOOST_RANDOM

IMP_ALGEBRA_HAS_BOOST_SYSTEM = _IMP_display.IMP_ALGEBRA_HAS_BOOST_SYSTEM

IMP_ALGEBRA_HAS_CGAL = _IMP_display.IMP_ALGEBRA_HAS_CGAL

IMP_ALGEBRA_HAS_NUMPY = _IMP_display.IMP_ALGEBRA_HAS_NUMPY

IMP_ALGEBRA_HAS_ANN = _IMP_display.IMP_ALGEBRA_HAS_ANN

IMPALGEBRA_SHOW_WARNINGS = _IMP_display.IMPALGEBRA_SHOW_WARNINGS

import IMP.algebra
IMP_DISPLAY_HAS_IMP_CGAL = _IMP_display.IMP_DISPLAY_HAS_IMP_CGAL

IMP_DISPLAY_HAS_BOOST_FILESYSTEM = _IMP_display.IMP_DISPLAY_HAS_BOOST_FILESYSTEM

IMP_DISPLAY_HAS_BOOST_PROGRAMOPTIONS = _IMP_display.IMP_DISPLAY_HAS_BOOST_PROGRAMOPTIONS

IMP_DISPLAY_HAS_BOOST_RANDOM = _IMP_display.IMP_DISPLAY_HAS_BOOST_RANDOM

IMP_DISPLAY_HAS_BOOST_SYSTEM = _IMP_display.IMP_DISPLAY_HAS_BOOST_SYSTEM

IMP_DISPLAY_HAS_CGAL = _IMP_display.IMP_DISPLAY_HAS_CGAL

IMP_DISPLAY_HAS_NUMPY = _IMP_display.IMP_DISPLAY_HAS_NUMPY

IMPDISPLAY_SHOW_WARNINGS = _IMP_display.IMPDISPLAY_SHOW_WARNINGS


_object_types.append("Writer")


def _object_cast_to_Writer(o):
    r"""_object_cast_to_Writer(Object o) -> Writer"""
    return _IMP_display._object_cast_to_Writer(o)

_object_types.append("TextWriter")


def _object_cast_to_TextWriter(o):
    r"""_object_cast_to_TextWriter(Object o) -> TextWriter"""
    return _IMP_display._object_cast_to_TextWriter(o)

_object_types.append("Geometry")


def _object_cast_to_Geometry(o):
    r"""_object_cast_to_Geometry(Object o) -> Geometry"""
    return _IMP_display._object_cast_to_Geometry(o)

_object_types.append("SingletonGeometry")


def _object_cast_to_SingletonGeometry(o):
    r"""_object_cast_to_SingletonGeometry(Object o) -> SingletonGeometry"""
    return _IMP_display._object_cast_to_SingletonGeometry(o)

_object_types.append("PairGeometry")


def _object_cast_to_PairGeometry(o):
    r"""_object_cast_to_PairGeometry(Object o) -> PairGeometry"""
    return _IMP_display._object_cast_to_PairGeometry(o)

_object_types.append("SingletonsGeometry")


def _object_cast_to_SingletonsGeometry(o):
    r"""_object_cast_to_SingletonsGeometry(Object o) -> SingletonsGeometry"""
    return _IMP_display._object_cast_to_SingletonsGeometry(o)

_object_types.append("PairsGeometry")


def _object_cast_to_PairsGeometry(o):
    r"""_object_cast_to_PairsGeometry(Object o) -> PairsGeometry"""
    return _IMP_display._object_cast_to_PairsGeometry(o)

_object_types.append("PymolWriter")


def _object_cast_to_PymolWriter(o):
    r"""_object_cast_to_PymolWriter(Object o) -> PymolWriter"""
    return _IMP_display._object_cast_to_PymolWriter(o)

_object_types.append("SphereGeometry")


def _object_cast_to_SphereGeometry(o):
    r"""_object_cast_to_SphereGeometry(Object o) -> SphereGeometry"""
    return _IMP_display._object_cast_to_SphereGeometry(o)

_object_types.append("CylinderGeometry")


def _object_cast_to_CylinderGeometry(o):
    r"""_object_cast_to_CylinderGeometry(Object o) -> CylinderGeometry"""
    return _IMP_display._object_cast_to_CylinderGeometry(o)

_object_types.append("EllipsoidGeometry")


def _object_cast_to_EllipsoidGeometry(o):
    r"""_object_cast_to_EllipsoidGeometry(Object o) -> EllipsoidGeometry"""
    return _IMP_display._object_cast_to_EllipsoidGeometry(o)

_object_types.append("PointGeometry")


def _object_cast_to_PointGeometry(o):
    r"""_object_cast_to_PointGeometry(Object o) -> PointGeometry"""
    return _IMP_display._object_cast_to_PointGeometry(o)

_object_types.append("SegmentGeometry")


def _object_cast_to_SegmentGeometry(o):
    r"""_object_cast_to_SegmentGeometry(Object o) -> SegmentGeometry"""
    return _IMP_display._object_cast_to_SegmentGeometry(o)

_object_types.append("PolygonGeometry")


def _object_cast_to_PolygonGeometry(o):
    r"""_object_cast_to_PolygonGeometry(Object o) -> PolygonGeometry"""
    return _IMP_display._object_cast_to_PolygonGeometry(o)

_object_types.append("ReferenceFrameGeometry")


def _object_cast_to_ReferenceFrameGeometry(o):
    r"""_object_cast_to_ReferenceFrameGeometry(Object o) -> ReferenceFrameGeometry"""
    return _IMP_display._object_cast_to_ReferenceFrameGeometry(o)

_object_types.append("BoundingBoxGeometry")


def _object_cast_to_BoundingBoxGeometry(o):
    r"""_object_cast_to_BoundingBoxGeometry(Object o) -> BoundingBoxGeometry"""
    return _IMP_display._object_cast_to_BoundingBoxGeometry(o)

_object_types.append("LabelGeometry")


def _object_cast_to_LabelGeometry(o):
    r"""_object_cast_to_LabelGeometry(Object o) -> LabelGeometry"""
    return _IMP_display._object_cast_to_LabelGeometry(o)

_object_types.append("RestraintGeometry")


def _object_cast_to_RestraintGeometry(o):
    r"""_object_cast_to_RestraintGeometry(Object o) -> RestraintGeometry"""
    return _IMP_display._object_cast_to_RestraintGeometry(o)

_object_types.append("RestraintSetGeometry")


def _object_cast_to_RestraintSetGeometry(o):
    r"""_object_cast_to_RestraintSetGeometry(Object o) -> RestraintSetGeometry"""
    return _IMP_display._object_cast_to_RestraintSetGeometry(o)

_object_types.append("GeometrySet")


def _object_cast_to_GeometrySet(o):
    r"""_object_cast_to_GeometrySet(Object o) -> GeometrySet"""
    return _IMP_display._object_cast_to_GeometrySet(o)

def Coloreds(l=[]):
    return [Colored(x) for x in l]
_plural_types.append("Coloreds")


_value_types.append("Colored")


Colors=list
_plural_types.append("Colors")
_value_types.append("Color")


WriterAdaptors=list
_plural_types.append("WriterAdaptors")
_value_types.append("WriterAdaptor")


_object_types.append("FilterGeometry")


def _object_cast_to_FilterGeometry(o):
    r"""_object_cast_to_FilterGeometry(Object o) -> FilterGeometry"""
    return _IMP_display._object_cast_to_FilterGeometry(o)

_object_types.append("WriteOptimizerState")


def _object_cast_to_WriteOptimizerState(o):
    r"""_object_cast_to_WriteOptimizerState(Object o) -> WriteOptimizerState"""
    return _IMP_display._object_cast_to_WriteOptimizerState(o)

_object_types.append("TriangleGeometry")


def _object_cast_to_TriangleGeometry(o):
    r"""_object_cast_to_TriangleGeometry(Object o) -> TriangleGeometry"""
    return _IMP_display._object_cast_to_TriangleGeometry(o)

_object_types.append("SurfaceMeshGeometry")


def _object_cast_to_SurfaceMeshGeometry(o):
    r"""_object_cast_to_SurfaceMeshGeometry(Object o) -> SurfaceMeshGeometry"""
    return _IMP_display._object_cast_to_SurfaceMeshGeometry(o)

_object_types.append("PlaneGeometry")


def _object_cast_to_PlaneGeometry(o):
    r"""_object_cast_to_PlaneGeometry(Object o) -> PlaneGeometry"""
    return _IMP_display._object_cast_to_PlaneGeometry(o)

_object_types.append("IsosurfaceGeometry")


def _object_cast_to_IsosurfaceGeometry(o):
    r"""_object_cast_to_IsosurfaceGeometry(Object o) -> IsosurfaceGeometry"""
    return _IMP_display._object_cast_to_IsosurfaceGeometry(o)

_object_types.append("SkinSurfaceGeometry")


def _object_cast_to_SkinSurfaceGeometry(o):
    r"""_object_cast_to_SkinSurfaceGeometry(Object o) -> SkinSurfaceGeometry"""
    return _IMP_display._object_cast_to_SkinSurfaceGeometry(o)
class Color(IMP._Value):
    r"""Proxy of C++ IMP::display::Color class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args):
        r"""
        __init__(Color self) -> Color
        __init__(Color self, double r, double g, double b) -> Color
        """
        _IMP_display.Color_swiginit(self, _IMP_display.new_Color(*args))
    __swig_destroy__ = _IMP_display.delete_Color

    def get_red(self):
        r"""get_red(Color self) -> double"""
        return _IMP_display.Color_get_red(self)

    def get_green(self):
        r"""get_green(Color self) -> double"""
        return _IMP_display.Color_get_green(self)

    def get_blue(self):
        r"""get_blue(Color self) -> double"""
        return _IMP_display.Color_get_blue(self)

    def show(self, *args):
        r"""
        show(Color self, _ostream out, std::string delim)
        show(Color self, _ostream out=std::cout)
        """
        return _IMP_display.Color_show(self, *args)

    def __cmp__(self, o):
        r"""__cmp__(Color self, Color o) -> int"""
        return _IMP_display.Color___cmp__(self, o)

    def __eq__(self, o):
        r"""__eq__(Color self, Color o) -> bool"""
        return _IMP_display.Color___eq__(self, o)

    def __ne__(self, o):
        r"""__ne__(Color self, Color o) -> bool"""
        return _IMP_display.Color___ne__(self, o)

    def __lt__(self, o):
        r"""__lt__(Color self, Color o) -> bool"""
        return _IMP_display.Color___lt__(self, o)

    def __gt__(self, o):
        r"""__gt__(Color self, Color o) -> bool"""
        return _IMP_display.Color___gt__(self, o)

    def __ge__(self, o):
        r"""__ge__(Color self, Color o) -> bool"""
        return _IMP_display.Color___ge__(self, o)

    def __le__(self, o):
        r"""__le__(Color self, Color o) -> bool"""
        return _IMP_display.Color___le__(self, o)

    def __str__(self):
        r"""__str__(Color self) -> std::string"""
        return _IMP_display.Color___str__(self)

    def __repr__(self):
        r"""__repr__(Color self) -> std::string"""
        return _IMP_display.Color___repr__(self)

# Register Color in _IMP_display:
_IMP_display.Color_swigregister(Color)


def get_display_color(i):
    r"""get_display_color(unsigned int i) -> Color"""
    return _IMP_display.get_display_color(i)

def get_interpolated_rgb(a, b, f):
    r"""get_interpolated_rgb(Color a, Color b, double f) -> Color"""
    return _IMP_display.get_interpolated_rgb(a, b, f)

def get_linear_color_map_value(min, max, value):
    r"""get_linear_color_map_value(double min, double max, double value) -> double"""
    return _IMP_display.get_linear_color_map_value(min, max, value)

def get_jet_color(f):
    r"""get_jet_color(double f) -> Color"""
    return _IMP_display.get_jet_color(f)

def get_hot_color(f):
    r"""get_hot_color(double f) -> Color"""
    return _IMP_display.get_hot_color(f)

def get_rgb_color(f):
    r"""get_rgb_color(double f) -> Color"""
    return _IMP_display.get_rgb_color(f)

def get_gray_color(f):
    r"""get_gray_color(double f) -> Color"""
    return _IMP_display.get_gray_color(f)

def get_gnuplot_color(f):
    r"""get_gnuplot_color(double f) -> Color"""
    return _IMP_display.get_gnuplot_color(f)
class Geometry(IMP.Object):
    r"""Proxy of C++ IMP::display::Geometry class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args):
        r"""
        __init__(Geometry self, std::string name) -> Geometry
        __init__(Geometry self, Color c, std::string name) -> Geometry
        """
        if self.__class__ == Geometry:
            _self = None
        else:
            _self = self
        _IMP_display.Geometry_swiginit(self, _IMP_display.new_Geometry(_self, *args))

        if self.__class__ != Geometry:
            _director_objects.register(self)




    def get_color(self):
        r"""get_color(Geometry self) -> Color"""
        return _IMP_display.Geometry_get_color(self)

    def get_has_color(self):
        r"""get_has_color(Geometry self) -> bool"""
        return _IMP_display.Geometry_get_has_color(self)

    def set_has_color(self, tf):
        r"""set_has_color(Geometry self, bool tf)"""
        return _IMP_display.Geometry_set_has_color(self, tf)

    def set_color(self, c):
        r"""set_color(Geometry self, Color c)"""
        return _IMP_display.Geometry_set_color(self, c)

    def get_components(self):
        r"""get_components(Geometry self) -> IMP::display::Geometries"""
        return _IMP_display.Geometry_get_components(self)
    __swig_destroy__ = _IMP_display.delete_Geometry

    def __str__(self):
        r"""__str__(Geometry self) -> std::string"""
        return _IMP_display.Geometry___str__(self)

    def __repr__(self):
        r"""__repr__(Geometry self) -> std::string"""
        return _IMP_display.Geometry___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_Geometry(o)


    def get_type_name(self):
        return self.__class__.__name__
    def do_show(self, out):
        pass
    def get_version_info(self):
        if"IMP::display" == "IMP":
          return VersionInfo(self.__module__,
                             __import__(self.__module__).get_module_version())
        else:
          return IMP.VersionInfo(self.__module__,
                             __import__(self.__module__).get_module_version())
    @staticmethod
    def get_from(o):
       return _object_cast_to_Geometry(o)

    def __disown__(self):
        self.this.disown()
        _IMP_display.disown_Geometry(self)
        return weakref.proxy(self)

    def do_destroy(self):
        r"""do_destroy(Geometry self)"""
        return _IMP_display.Geometry_do_destroy(self)

# Register Geometry in _IMP_display:
_IMP_display.Geometry_swigregister(Geometry)

class GeometrySet(Geometry):
    r"""Proxy of C++ IMP::display::GeometrySet class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args):
        r"""
        __init__(GeometrySet self, IMP::display::Geometries const & v) -> GeometrySet
        __init__(GeometrySet self, IMP::display::Geometries const & v, Color c) -> GeometrySet
        __init__(GeometrySet self, IMP::display::Geometries const & v, std::string const n) -> GeometrySet
        __init__(GeometrySet self, IMP::display::Geometries const & v, Color c, std::string n) -> GeometrySet
        """
        _IMP_display.GeometrySet_swiginit(self, _IMP_display.new_GeometrySet(*args))

    def get_geometry(self):
        r"""get_geometry(GeometrySet self) -> IMP::display::Geometries"""
        return _IMP_display.GeometrySet_get_geometry(self)

    def get_version_info(self):
        r"""get_version_info(GeometrySet self) -> VersionInfo"""
        return _IMP_display.GeometrySet_get_version_info(self)

    def __str__(self):
        r"""__str__(GeometrySet self) -> std::string"""
        return _IMP_display.GeometrySet___str__(self)

    def __repr__(self):
        r"""__repr__(GeometrySet self) -> std::string"""
        return _IMP_display.GeometrySet___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_GeometrySet(o)


# Register GeometrySet in _IMP_display:
_IMP_display.GeometrySet_swigregister(GeometrySet)

class SphereGeometry(Geometry):
    r"""Proxy of C++ IMP::display::SphereGeometry class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args):
        r"""
        __init__(SphereGeometry self, Sphere3D v) -> SphereGeometry
        __init__(SphereGeometry self, Sphere3D v, Color c) -> SphereGeometry
        __init__(SphereGeometry self, Sphere3D v, std::string const n) -> SphereGeometry
        __init__(SphereGeometry self, Sphere3D v, Color c, std::string n) -> SphereGeometry
        """
        _IMP_display.SphereGeometry_swiginit(self, _IMP_display.new_SphereGeometry(*args))

    def get_geometry(self):
        r"""get_geometry(SphereGeometry self) -> Sphere3D"""
        return _IMP_display.SphereGeometry_get_geometry(self)

    def set_geometry(self, v):
        r"""set_geometry(SphereGeometry self, Sphere3D v)"""
        return _IMP_display.SphereGeometry_set_geometry(self, v)

    def get_version_info(self):
        r"""get_version_info(SphereGeometry self) -> VersionInfo"""
        return _IMP_display.SphereGeometry_get_version_info(self)

    def __str__(self):
        r"""__str__(SphereGeometry self) -> std::string"""
        return _IMP_display.SphereGeometry___str__(self)

    def __repr__(self):
        r"""__repr__(SphereGeometry self) -> std::string"""
        return _IMP_display.SphereGeometry___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_SphereGeometry(o)


# Register SphereGeometry in _IMP_display:
_IMP_display.SphereGeometry_swigregister(SphereGeometry)

class CylinderGeometry(Geometry):
    r"""Proxy of C++ IMP::display::CylinderGeometry class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args):
        r"""
        __init__(CylinderGeometry self, Cylinder3D v) -> CylinderGeometry
        __init__(CylinderGeometry self, Cylinder3D v, Color c) -> CylinderGeometry
        __init__(CylinderGeometry self, Cylinder3D v, std::string const n) -> CylinderGeometry
        __init__(CylinderGeometry self, Cylinder3D v, Color c, std::string n) -> CylinderGeometry
        """
        _IMP_display.CylinderGeometry_swiginit(self, _IMP_display.new_CylinderGeometry(*args))

    def get_geometry(self):
        r"""get_geometry(CylinderGeometry self) -> Cylinder3D"""
        return _IMP_display.CylinderGeometry_get_geometry(self)

    def set_geometry(self, v):
        r"""set_geometry(CylinderGeometry self, Cylinder3D v)"""
        return _IMP_display.CylinderGeometry_set_geometry(self, v)

    def get_version_info(self):
        r"""get_version_info(CylinderGeometry self) -> VersionInfo"""
        return _IMP_display.CylinderGeometry_get_version_info(self)

    def __str__(self):
        r"""__str__(CylinderGeometry self) -> std::string"""
        return _IMP_display.CylinderGeometry___str__(self)

    def __repr__(self):
        r"""__repr__(CylinderGeometry self) -> std::string"""
        return _IMP_display.CylinderGeometry___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_CylinderGeometry(o)


# Register CylinderGeometry in _IMP_display:
_IMP_display.CylinderGeometry_swigregister(CylinderGeometry)

class EllipsoidGeometry(Geometry):
    r"""Proxy of C++ IMP::display::EllipsoidGeometry class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args):
        r"""
        __init__(EllipsoidGeometry self, Ellipsoid3D v) -> EllipsoidGeometry
        __init__(EllipsoidGeometry self, Ellipsoid3D v, Color c) -> EllipsoidGeometry
        __init__(EllipsoidGeometry self, Ellipsoid3D v, std::string const n) -> EllipsoidGeometry
        __init__(EllipsoidGeometry self, Ellipsoid3D v, Color c, std::string n) -> EllipsoidGeometry
        """
        _IMP_display.EllipsoidGeometry_swiginit(self, _IMP_display.new_EllipsoidGeometry(*args))

    def get_geometry(self):
        r"""get_geometry(EllipsoidGeometry self) -> Ellipsoid3D"""
        return _IMP_display.EllipsoidGeometry_get_geometry(self)

    def set_geometry(self, v):
        r"""set_geometry(EllipsoidGeometry self, Ellipsoid3D v)"""
        return _IMP_display.EllipsoidGeometry_set_geometry(self, v)

    def get_version_info(self):
        r"""get_version_info(EllipsoidGeometry self) -> VersionInfo"""
        return _IMP_display.EllipsoidGeometry_get_version_info(self)

    def __str__(self):
        r"""__str__(EllipsoidGeometry self) -> std::string"""
        return _IMP_display.EllipsoidGeometry___str__(self)

    def __repr__(self):
        r"""__repr__(EllipsoidGeometry self) -> std::string"""
        return _IMP_display.EllipsoidGeometry___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_EllipsoidGeometry(o)


# Register EllipsoidGeometry in _IMP_display:
_IMP_display.EllipsoidGeometry_swigregister(EllipsoidGeometry)

class PointGeometry(Geometry):
    r"""Proxy of C++ IMP::display::PointGeometry class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args):
        r"""
        __init__(PointGeometry self, Vector3D v) -> PointGeometry
        __init__(PointGeometry self, Vector3D v, Color c) -> PointGeometry
        __init__(PointGeometry self, Vector3D v, std::string const n) -> PointGeometry
        __init__(PointGeometry self, Vector3D v, Color c, std::string n) -> PointGeometry
        """
        _IMP_display.PointGeometry_swiginit(self, _IMP_display.new_PointGeometry(*args))

    def get_geometry(self):
        r"""get_geometry(PointGeometry self) -> Vector3D"""
        return _IMP_display.PointGeometry_get_geometry(self)

    def set_geometry(self, v):
        r"""set_geometry(PointGeometry self, Vector3D v)"""
        return _IMP_display.PointGeometry_set_geometry(self, v)

    def get_version_info(self):
        r"""get_version_info(PointGeometry self) -> VersionInfo"""
        return _IMP_display.PointGeometry_get_version_info(self)

    def __str__(self):
        r"""__str__(PointGeometry self) -> std::string"""
        return _IMP_display.PointGeometry___str__(self)

    def __repr__(self):
        r"""__repr__(PointGeometry self) -> std::string"""
        return _IMP_display.PointGeometry___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_PointGeometry(o)


# Register PointGeometry in _IMP_display:
_IMP_display.PointGeometry_swigregister(PointGeometry)

class SegmentGeometry(Geometry):
    r"""Proxy of C++ IMP::display::SegmentGeometry class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args):
        r"""
        __init__(SegmentGeometry self, Segment3D v) -> SegmentGeometry
        __init__(SegmentGeometry self, Segment3D v, Color c) -> SegmentGeometry
        __init__(SegmentGeometry self, Segment3D v, std::string const n) -> SegmentGeometry
        __init__(SegmentGeometry self, Segment3D v, Color c, std::string n) -> SegmentGeometry
        """
        _IMP_display.SegmentGeometry_swiginit(self, _IMP_display.new_SegmentGeometry(*args))

    def get_geometry(self):
        r"""get_geometry(SegmentGeometry self) -> Segment3D"""
        return _IMP_display.SegmentGeometry_get_geometry(self)

    def set_geometry(self, v):
        r"""set_geometry(SegmentGeometry self, Segment3D v)"""
        return _IMP_display.SegmentGeometry_set_geometry(self, v)

    def get_version_info(self):
        r"""get_version_info(SegmentGeometry self) -> VersionInfo"""
        return _IMP_display.SegmentGeometry_get_version_info(self)

    def __str__(self):
        r"""__str__(SegmentGeometry self) -> std::string"""
        return _IMP_display.SegmentGeometry___str__(self)

    def __repr__(self):
        r"""__repr__(SegmentGeometry self) -> std::string"""
        return _IMP_display.SegmentGeometry___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_SegmentGeometry(o)


# Register SegmentGeometry in _IMP_display:
_IMP_display.SegmentGeometry_swigregister(SegmentGeometry)

class PolygonGeometry(Geometry):
    r"""Proxy of C++ IMP::display::PolygonGeometry class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args):
        r"""
        __init__(PolygonGeometry self, IMP::algebra::Vector3Ds const & v) -> PolygonGeometry
        __init__(PolygonGeometry self, IMP::algebra::Vector3Ds const & v, Color c) -> PolygonGeometry
        __init__(PolygonGeometry self, IMP::algebra::Vector3Ds const & v, std::string const n) -> PolygonGeometry
        __init__(PolygonGeometry self, IMP::algebra::Vector3Ds const & v, Color c, std::string n) -> PolygonGeometry
        """
        _IMP_display.PolygonGeometry_swiginit(self, _IMP_display.new_PolygonGeometry(*args))

    def get_geometry(self):
        r"""get_geometry(PolygonGeometry self) -> IMP::algebra::Vector3Ds const &"""
        return _IMP_display.PolygonGeometry_get_geometry(self)

    def set_geometry(self, v):
        r"""set_geometry(PolygonGeometry self, IMP::algebra::Vector3Ds const & v)"""
        return _IMP_display.PolygonGeometry_set_geometry(self, v)

    def get_version_info(self):
        r"""get_version_info(PolygonGeometry self) -> VersionInfo"""
        return _IMP_display.PolygonGeometry_get_version_info(self)

    def __str__(self):
        r"""__str__(PolygonGeometry self) -> std::string"""
        return _IMP_display.PolygonGeometry___str__(self)

    def __repr__(self):
        r"""__repr__(PolygonGeometry self) -> std::string"""
        return _IMP_display.PolygonGeometry___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_PolygonGeometry(o)


# Register PolygonGeometry in _IMP_display:
_IMP_display.PolygonGeometry_swigregister(PolygonGeometry)

class TriangleGeometry(Geometry):
    r"""Proxy of C++ IMP::display::TriangleGeometry class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args):
        r"""
        __init__(TriangleGeometry self, Triangle3D v) -> TriangleGeometry
        __init__(TriangleGeometry self, Triangle3D v, Color c) -> TriangleGeometry
        __init__(TriangleGeometry self, Triangle3D v, std::string const n) -> TriangleGeometry
        __init__(TriangleGeometry self, Triangle3D v, Color c, std::string n) -> TriangleGeometry
        """
        _IMP_display.TriangleGeometry_swiginit(self, _IMP_display.new_TriangleGeometry(*args))

    def get_geometry(self):
        r"""get_geometry(TriangleGeometry self) -> Triangle3D"""
        return _IMP_display.TriangleGeometry_get_geometry(self)

    def set_geometry(self, v):
        r"""set_geometry(TriangleGeometry self, Triangle3D v)"""
        return _IMP_display.TriangleGeometry_set_geometry(self, v)

    def get_version_info(self):
        r"""get_version_info(TriangleGeometry self) -> VersionInfo"""
        return _IMP_display.TriangleGeometry_get_version_info(self)

    def __str__(self):
        r"""__str__(TriangleGeometry self) -> std::string"""
        return _IMP_display.TriangleGeometry___str__(self)

    def __repr__(self):
        r"""__repr__(TriangleGeometry self) -> std::string"""
        return _IMP_display.TriangleGeometry___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_TriangleGeometry(o)


# Register TriangleGeometry in _IMP_display:
_IMP_display.TriangleGeometry_swigregister(TriangleGeometry)

class BoundingBoxGeometry(Geometry):
    r"""Proxy of C++ IMP::display::BoundingBoxGeometry class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args):
        r"""
        __init__(BoundingBoxGeometry self, BoundingBox3D v) -> BoundingBoxGeometry
        __init__(BoundingBoxGeometry self, BoundingBox3D v, Color c) -> BoundingBoxGeometry
        __init__(BoundingBoxGeometry self, BoundingBox3D v, std::string const n) -> BoundingBoxGeometry
        __init__(BoundingBoxGeometry self, BoundingBox3D v, Color c, std::string n) -> BoundingBoxGeometry
        """
        _IMP_display.BoundingBoxGeometry_swiginit(self, _IMP_display.new_BoundingBoxGeometry(*args))

    def get_geometry(self):
        r"""get_geometry(BoundingBoxGeometry self) -> BoundingBox3D"""
        return _IMP_display.BoundingBoxGeometry_get_geometry(self)

    def set_geometry(self, v):
        r"""set_geometry(BoundingBoxGeometry self, BoundingBox3D v)"""
        return _IMP_display.BoundingBoxGeometry_set_geometry(self, v)

    def get_version_info(self):
        r"""get_version_info(BoundingBoxGeometry self) -> VersionInfo"""
        return _IMP_display.BoundingBoxGeometry_get_version_info(self)

    def __str__(self):
        r"""__str__(BoundingBoxGeometry self) -> std::string"""
        return _IMP_display.BoundingBoxGeometry___str__(self)

    def __repr__(self):
        r"""__repr__(BoundingBoxGeometry self) -> std::string"""
        return _IMP_display.BoundingBoxGeometry___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_BoundingBoxGeometry(o)


# Register BoundingBoxGeometry in _IMP_display:
_IMP_display.BoundingBoxGeometry_swigregister(BoundingBoxGeometry)

class ReferenceFrameGeometry(Geometry):
    r"""Proxy of C++ IMP::display::ReferenceFrameGeometry class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args):
        r"""
        __init__(ReferenceFrameGeometry self, ReferenceFrame3D v) -> ReferenceFrameGeometry
        __init__(ReferenceFrameGeometry self, ReferenceFrame3D v, Color c) -> ReferenceFrameGeometry
        __init__(ReferenceFrameGeometry self, ReferenceFrame3D v, std::string const n) -> ReferenceFrameGeometry
        __init__(ReferenceFrameGeometry self, ReferenceFrame3D v, Color c, std::string n) -> ReferenceFrameGeometry
        """
        _IMP_display.ReferenceFrameGeometry_swiginit(self, _IMP_display.new_ReferenceFrameGeometry(*args))

    def get_geometry(self):
        r"""get_geometry(ReferenceFrameGeometry self) -> ReferenceFrame3D"""
        return _IMP_display.ReferenceFrameGeometry_get_geometry(self)

    def set_geometry(self, v):
        r"""set_geometry(ReferenceFrameGeometry self, ReferenceFrame3D v)"""
        return _IMP_display.ReferenceFrameGeometry_set_geometry(self, v)

    def get_version_info(self):
        r"""get_version_info(ReferenceFrameGeometry self) -> VersionInfo"""
        return _IMP_display.ReferenceFrameGeometry_get_version_info(self)

    def __str__(self):
        r"""__str__(ReferenceFrameGeometry self) -> std::string"""
        return _IMP_display.ReferenceFrameGeometry___str__(self)

    def __repr__(self):
        r"""__repr__(ReferenceFrameGeometry self) -> std::string"""
        return _IMP_display.ReferenceFrameGeometry___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_ReferenceFrameGeometry(o)


# Register ReferenceFrameGeometry in _IMP_display:
_IMP_display.ReferenceFrameGeometry_swigregister(ReferenceFrameGeometry)


def create_geometry(*args):
    r"""
    create_geometry(Sphere3D t, std::string name="algebra::Sphere3D+%1%") -> SphereGeometry
    create_geometry(Cylinder3D t, std::string name="algebra::Cylinder3D+%1%") -> CylinderGeometry
    create_geometry(Ellipsoid3D t, std::string name="algebra::Ellipsoid3D+%1%") -> EllipsoidGeometry
    create_geometry(Vector3D t, std::string name="algebra::Vector3D+%1%") -> PointGeometry
    create_geometry(Segment3D t, std::string name="algebra::Segment3D+%1%") -> SegmentGeometry
    create_geometry(IMP::algebra::Vector3Ds const & t, std::string name="algebra::Vector3Ds+%1%") -> PolygonGeometry
    create_geometry(Triangle3D t, std::string name="algebra::Triangle3D+%1%") -> TriangleGeometry
    create_geometry(BoundingBox3D t, std::string name="algebra::BoundingBoxD<3>+%1%") -> BoundingBoxGeometry
    create_geometry(ReferenceFrame3D t, std::string name="algebra::ReferenceFrame3D+%1%") -> ReferenceFrameGeometry
    """
    return _IMP_display.create_geometry(*args)
class LabelGeometry(Geometry):
    r"""Proxy of C++ IMP::display::LabelGeometry class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args):
        r"""
        __init__(LabelGeometry self, Sphere3D loc, std::string text) -> LabelGeometry
        __init__(LabelGeometry self, Vector3D loc, std::string text) -> LabelGeometry
        """
        _IMP_display.LabelGeometry_swiginit(self, _IMP_display.new_LabelGeometry(*args))

    def get_text(self):
        r"""get_text(LabelGeometry self) -> std::string"""
        return _IMP_display.LabelGeometry_get_text(self)

    def get_location(self):
        r"""get_location(LabelGeometry self) -> Sphere3D"""
        return _IMP_display.LabelGeometry_get_location(self)

    def get_version_info(self):
        r"""get_version_info(LabelGeometry self) -> VersionInfo"""
        return _IMP_display.LabelGeometry_get_version_info(self)

    def __str__(self):
        r"""__str__(LabelGeometry self) -> std::string"""
        return _IMP_display.LabelGeometry___str__(self)

    def __repr__(self):
        r"""__repr__(LabelGeometry self) -> std::string"""
        return _IMP_display.LabelGeometry___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_LabelGeometry(o)


# Register LabelGeometry in _IMP_display:
_IMP_display.LabelGeometry_swigregister(LabelGeometry)

class SurfaceMeshGeometry(Geometry):
    r"""Proxy of C++ IMP::display::SurfaceMeshGeometry class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, vertices, faces):
        r"""__init__(SurfaceMeshGeometry self, IMP::algebra::Vector3Ds const & vertices, IMP::Ints const & faces) -> SurfaceMeshGeometry"""
        _IMP_display.SurfaceMeshGeometry_swiginit(self, _IMP_display.new_SurfaceMeshGeometry(vertices, faces))

    def get_vertexes(self):
        r"""get_vertexes(SurfaceMeshGeometry self) -> IMP::algebra::Vector3Ds const &"""
        return _IMP_display.SurfaceMeshGeometry_get_vertexes(self)

    def get_faces(self):
        r"""get_faces(SurfaceMeshGeometry self) -> IMP::Ints const &"""
        return _IMP_display.SurfaceMeshGeometry_get_faces(self)

    def get_version_info(self):
        r"""get_version_info(SurfaceMeshGeometry self) -> VersionInfo"""
        return _IMP_display.SurfaceMeshGeometry_get_version_info(self)

    def __str__(self):
        r"""__str__(SurfaceMeshGeometry self) -> std::string"""
        return _IMP_display.SurfaceMeshGeometry___str__(self)

    def __repr__(self):
        r"""__repr__(SurfaceMeshGeometry self) -> std::string"""
        return _IMP_display.SurfaceMeshGeometry___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_SurfaceMeshGeometry(o)


# Register SurfaceMeshGeometry in _IMP_display:
_IMP_display.SurfaceMeshGeometry_swigregister(SurfaceMeshGeometry)

class PlaneGeometry(Geometry):
    r"""Proxy of C++ IMP::display::PlaneGeometry class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, loc, box):
        r"""__init__(PlaneGeometry self, Plane3D loc, BoundingBox3D box) -> PlaneGeometry"""
        _IMP_display.PlaneGeometry_swiginit(self, _IMP_display.new_PlaneGeometry(loc, box))

    def get_version_info(self):
        r"""get_version_info(PlaneGeometry self) -> VersionInfo"""
        return _IMP_display.PlaneGeometry_get_version_info(self)

    def __str__(self):
        r"""__str__(PlaneGeometry self) -> std::string"""
        return _IMP_display.PlaneGeometry___str__(self)

    def __repr__(self):
        r"""__repr__(PlaneGeometry self) -> std::string"""
        return _IMP_display.PlaneGeometry___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_PlaneGeometry(o)


# Register PlaneGeometry in _IMP_display:
_IMP_display.PlaneGeometry_swigregister(PlaneGeometry)

class IsosurfaceGeometry(SurfaceMeshGeometry):
    r"""Proxy of C++ IMP::display::IsosurfaceGeometry class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args):
        r"""
        __init__(IsosurfaceGeometry self, IMP::algebra::GridD< 3,IMP::algebra::DenseGridStorageD< 3,double >,double,IMP::algebra::DefaultEmbeddingD< 3 > > const & grid, double iso) -> IsosurfaceGeometry
        __init__(IsosurfaceGeometry self, IMP::algebra::GridD< 3,IMP::algebra::DenseGridStorageD< 3,float >,float,IMP::algebra::DefaultEmbeddingD< 3 > > const & grid, double iso) -> IsosurfaceGeometry
        """
        _IMP_display.IsosurfaceGeometry_swiginit(self, _IMP_display.new_IsosurfaceGeometry(*args))

    def __str__(self):
        r"""__str__(IsosurfaceGeometry self) -> std::string"""
        return _IMP_display.IsosurfaceGeometry___str__(self)

    def __repr__(self):
        r"""__repr__(IsosurfaceGeometry self) -> std::string"""
        return _IMP_display.IsosurfaceGeometry___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_IsosurfaceGeometry(o)

    __swig_destroy__ = _IMP_display.delete_IsosurfaceGeometry

# Register IsosurfaceGeometry in _IMP_display:
_IMP_display.IsosurfaceGeometry_swigregister(IsosurfaceGeometry)

class SkinSurfaceGeometry(SurfaceMeshGeometry):
    r"""Proxy of C++ IMP::display::SkinSurfaceGeometry class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, balls):
        r"""__init__(SkinSurfaceGeometry self, IMP::algebra::Sphere3Ds const & balls) -> SkinSurfaceGeometry"""
        _IMP_display.SkinSurfaceGeometry_swiginit(self, _IMP_display.new_SkinSurfaceGeometry(balls))

    def __str__(self):
        r"""__str__(SkinSurfaceGeometry self) -> std::string"""
        return _IMP_display.SkinSurfaceGeometry___str__(self)

    def __repr__(self):
        r"""__repr__(SkinSurfaceGeometry self) -> std::string"""
        return _IMP_display.SkinSurfaceGeometry___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_SkinSurfaceGeometry(o)

    __swig_destroy__ = _IMP_display.delete_SkinSurfaceGeometry

# Register SkinSurfaceGeometry in _IMP_display:
_IMP_display.SkinSurfaceGeometry_swigregister(SkinSurfaceGeometry)

class GeometryProcessor(object):
    r"""Proxy of C++ IMP::display::GeometryProcessor class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self):
        r"""__init__(GeometryProcessor self) -> GeometryProcessor"""
        _IMP_display.GeometryProcessor_swiginit(self, _IMP_display.new_GeometryProcessor())

# Register GeometryProcessor in _IMP_display:
_IMP_display.GeometryProcessor_swigregister(GeometryProcessor)

class Writer(GeometryProcessor, IMP.Object):
    r"""Proxy of C++ IMP::display::Writer class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, name):
        r"""__init__(Writer self, std::string name) -> Writer"""
        if self.__class__ == Writer:
            _self = None
        else:
            _self = self
        _IMP_display.Writer_swiginit(self, _IMP_display.new_Writer(_self, name))

        if self.__class__ != Writer:
            _director_objects.register(self)



    __swig_destroy__ = _IMP_display.delete_Writer

    def set_frame(self, i):
        r"""set_frame(Writer self, unsigned int i)"""
        return _IMP_display.Writer_set_frame(self, i)

    def get_frame(self):
        r"""get_frame(Writer self) -> int"""
        return _IMP_display.Writer_get_frame(self)

    def add_geometry(self, *args):
        r"""
        add_geometry(Writer self, Geometry g)
        add_geometry(Writer self, IMP::display::Geometries const & g)
        """
        return _IMP_display.Writer_add_geometry(self, *args)

    def do_close(self):
        r"""do_close(Writer self)"""
        return _IMP_display.Writer_do_close(self)

    def do_open(self):
        r"""do_open(Writer self)"""
        return _IMP_display.Writer_do_open(self)

    def do_set_frame(self):
        r"""do_set_frame(Writer self)"""
        return _IMP_display.Writer_do_set_frame(self)

    def __str__(self):
        r"""__str__(Writer self) -> std::string"""
        return _IMP_display.Writer___str__(self)

    def __repr__(self):
        r"""__repr__(Writer self) -> std::string"""
        return _IMP_display.Writer___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_Writer(o)


    def get_type_name(self):
        return self.__class__.__name__
    def do_show(self, out):
        pass
    def get_version_info(self):
        if"IMP::display" == "IMP":
          return VersionInfo(self.__module__,
                             __import__(self.__module__).get_module_version())
        else:
          return IMP.VersionInfo(self.__module__,
                             __import__(self.__module__).get_module_version())
    @staticmethod
    def get_from(o):
       return _object_cast_to_Writer(o)

    def __disown__(self):
        self.this.disown()
        _IMP_display.disown_Writer(self)
        return weakref.proxy(self)

    def handle_sphere(self, arg0, arg1, arg2):
        r"""handle_sphere(Writer self, SphereGeometry arg0, Color arg1, std::string arg2) -> bool"""
        return _IMP_display.Writer_handle_sphere(self, arg0, arg1, arg2)

    def handle_cylinder(self, arg0, arg1, arg2):
        r"""handle_cylinder(Writer self, CylinderGeometry arg0, Color arg1, std::string arg2) -> bool"""
        return _IMP_display.Writer_handle_cylinder(self, arg0, arg1, arg2)

    def handle_point(self, arg0, arg1, arg2):
        r"""handle_point(Writer self, PointGeometry arg0, Color arg1, std::string arg2) -> bool"""
        return _IMP_display.Writer_handle_point(self, arg0, arg1, arg2)

    def handle_segment(self, arg0, arg1, arg2):
        r"""handle_segment(Writer self, SegmentGeometry arg0, Color arg1, std::string arg2) -> bool"""
        return _IMP_display.Writer_handle_segment(self, arg0, arg1, arg2)

    def handle_polygon(self, arg0, arg1, arg2):
        r"""handle_polygon(Writer self, PolygonGeometry arg0, Color arg1, std::string arg2) -> bool"""
        return _IMP_display.Writer_handle_polygon(self, arg0, arg1, arg2)

    def handle_triangle(self, arg0, arg1, arg2):
        r"""handle_triangle(Writer self, TriangleGeometry arg0, Color arg1, std::string arg2) -> bool"""
        return _IMP_display.Writer_handle_triangle(self, arg0, arg1, arg2)

    def handle_surface_mesh(self, arg0, arg1, arg2):
        r"""handle_surface_mesh(Writer self, SurfaceMeshGeometry arg0, Color arg1, std::string arg2) -> bool"""
        return _IMP_display.Writer_handle_surface_mesh(self, arg0, arg1, arg2)

    def handle_label(self, arg0, arg1, arg2):
        r"""handle_label(Writer self, LabelGeometry arg0, Color arg1, std::string arg2) -> bool"""
        return _IMP_display.Writer_handle_label(self, arg0, arg1, arg2)

    def handle_ellipsoid(self, arg0, arg1, arg2):
        r"""handle_ellipsoid(Writer self, EllipsoidGeometry arg0, Color arg1, std::string arg2) -> bool"""
        return _IMP_display.Writer_handle_ellipsoid(self, arg0, arg1, arg2)

    def handle_bounding_box(self, arg0, arg1, arg2):
        r"""handle_bounding_box(Writer self, BoundingBoxGeometry arg0, Color arg1, std::string arg2) -> bool"""
        return _IMP_display.Writer_handle_bounding_box(self, arg0, arg1, arg2)

    def handle_anything(self, arg0, arg1, arg2):
        r"""handle_anything(Writer self, Geometry arg0, Color arg1, std::string arg2) -> bool"""
        return _IMP_display.Writer_handle_anything(self, arg0, arg1, arg2)

    def do_destroy(self):
        r"""do_destroy(Writer self)"""
        return _IMP_display.Writer_do_destroy(self)

# Register Writer in _IMP_display:
_IMP_display.Writer_swigregister(Writer)

class TextWriter(Writer):
    r"""Proxy of C++ IMP::display::TextWriter class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def open(self):
        r"""open(TextWriter self)"""
        return _IMP_display.TextWriter_open(self)

    def get_stream(self):
        r"""get_stream(TextWriter self) -> _ostream"""
        return _IMP_display.TextWriter_get_stream(self)

    def do_set_frame(self):
        r"""do_set_frame(TextWriter self)"""
        return _IMP_display.TextWriter_do_set_frame(self)

    def __init__(self, *args):
        r"""
        __init__(TextWriter self, TextOutput fn) -> TextWriter
        __init__(TextWriter self, std::string name) -> TextWriter
        """
        if self.__class__ == TextWriter:
            _self = None
        else:
            _self = self
        _IMP_display.TextWriter_swiginit(self, _IMP_display.new_TextWriter(_self, *args))

        if self.__class__ != TextWriter:
            _director_objects.register(self)




    def get_current_file_name(self):
        r"""get_current_file_name(TextWriter self) -> std::string"""
        return _IMP_display.TextWriter_get_current_file_name(self)
    __swig_destroy__ = _IMP_display.delete_TextWriter

    def __str__(self):
        r"""__str__(TextWriter self) -> std::string"""
        return _IMP_display.TextWriter___str__(self)

    def __repr__(self):
        r"""__repr__(TextWriter self) -> std::string"""
        return _IMP_display.TextWriter___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_TextWriter(o)


    def get_type_name(self):
        return self.__class__.__name__
    def do_show(self, out):
        pass
    def get_version_info(self):
        if"IMP::display" == "IMP":
          return VersionInfo(self.__module__,
                             __import__(self.__module__).get_module_version())
        else:
          return IMP.VersionInfo(self.__module__,
                             __import__(self.__module__).get_module_version())
    @staticmethod
    def get_from(o):
       return _object_cast_to_TextWriter(o)

    def __disown__(self):
        self.this.disown()
        _IMP_display.disown_TextWriter(self)
        return weakref.proxy(self)

    def handle_sphere(self, arg0, arg1, arg2):
        r"""handle_sphere(TextWriter self, SphereGeometry arg0, Color arg1, std::string arg2) -> bool"""
        return _IMP_display.TextWriter_handle_sphere(self, arg0, arg1, arg2)

    def handle_cylinder(self, arg0, arg1, arg2):
        r"""handle_cylinder(TextWriter self, CylinderGeometry arg0, Color arg1, std::string arg2) -> bool"""
        return _IMP_display.TextWriter_handle_cylinder(self, arg0, arg1, arg2)

    def handle_point(self, arg0, arg1, arg2):
        r"""handle_point(TextWriter self, PointGeometry arg0, Color arg1, std::string arg2) -> bool"""
        return _IMP_display.TextWriter_handle_point(self, arg0, arg1, arg2)

    def handle_segment(self, arg0, arg1, arg2):
        r"""handle_segment(TextWriter self, SegmentGeometry arg0, Color arg1, std::string arg2) -> bool"""
        return _IMP_display.TextWriter_handle_segment(self, arg0, arg1, arg2)

    def handle_polygon(self, arg0, arg1, arg2):
        r"""handle_polygon(TextWriter self, PolygonGeometry arg0, Color arg1, std::string arg2) -> bool"""
        return _IMP_display.TextWriter_handle_polygon(self, arg0, arg1, arg2)

    def handle_triangle(self, arg0, arg1, arg2):
        r"""handle_triangle(TextWriter self, TriangleGeometry arg0, Color arg1, std::string arg2) -> bool"""
        return _IMP_display.TextWriter_handle_triangle(self, arg0, arg1, arg2)

    def handle_surface_mesh(self, arg0, arg1, arg2):
        r"""handle_surface_mesh(TextWriter self, SurfaceMeshGeometry arg0, Color arg1, std::string arg2) -> bool"""
        return _IMP_display.TextWriter_handle_surface_mesh(self, arg0, arg1, arg2)

    def handle_label(self, arg0, arg1, arg2):
        r"""handle_label(TextWriter self, LabelGeometry arg0, Color arg1, std::string arg2) -> bool"""
        return _IMP_display.TextWriter_handle_label(self, arg0, arg1, arg2)

    def handle_ellipsoid(self, arg0, arg1, arg2):
        r"""handle_ellipsoid(TextWriter self, EllipsoidGeometry arg0, Color arg1, std::string arg2) -> bool"""
        return _IMP_display.TextWriter_handle_ellipsoid(self, arg0, arg1, arg2)

    def handle_bounding_box(self, arg0, arg1, arg2):
        r"""handle_bounding_box(TextWriter self, BoundingBoxGeometry arg0, Color arg1, std::string arg2) -> bool"""
        return _IMP_display.TextWriter_handle_bounding_box(self, arg0, arg1, arg2)

    def handle_anything(self, arg0, arg1, arg2):
        r"""handle_anything(TextWriter self, Geometry arg0, Color arg1, std::string arg2) -> bool"""
        return _IMP_display.TextWriter_handle_anything(self, arg0, arg1, arg2)

    def do_destroy(self):
        r"""do_destroy(TextWriter self)"""
        return _IMP_display.TextWriter_do_destroy(self)

    def do_close(self):
        r"""do_close(TextWriter self)"""
        return _IMP_display.TextWriter_do_close(self)

    def do_open(self):
        r"""do_open(TextWriter self)"""
        return _IMP_display.TextWriter_do_open(self)

# Register TextWriter in _IMP_display:
_IMP_display.TextWriter_swigregister(TextWriter)


def create_writer(filename):
    r"""create_writer(std::string filename) -> Writer"""
    return _IMP_display.create_writer(filename)
class WriterAdaptor(IMP._InputAdaptor):
    r"""Proxy of C++ IMP::display::WriterAdaptor class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args):
        r"""
        __init__(WriterAdaptor self, std::string name) -> WriterAdaptor
        __init__(WriterAdaptor self, Writer w) -> WriterAdaptor
        """
        _IMP_display.WriterAdaptor_swiginit(self, _IMP_display.new_WriterAdaptor(*args))

    def get_writer(self):
        r"""get_writer(WriterAdaptor self) -> Writer"""
        return _IMP_display.WriterAdaptor_get_writer(self)

    def show(self, *args):
        r"""show(WriterAdaptor self, _ostream out=std::cout)"""
        return _IMP_display.WriterAdaptor_show(self, *args)
    __swig_destroy__ = _IMP_display.delete_WriterAdaptor

    def __str__(self):
        r"""__str__(WriterAdaptor self) -> std::string"""
        return _IMP_display.WriterAdaptor___str__(self)

    def __repr__(self):
        r"""__repr__(WriterAdaptor self) -> std::string"""
        return _IMP_display.WriterAdaptor___repr__(self)

# Register WriterAdaptor in _IMP_display:
_IMP_display.WriterAdaptor_swigregister(WriterAdaptor)

class Colored(IMP.Decorator):
    r"""Proxy of C++ IMP::display::Colored class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def set_color(self, c):
        r"""set_color(Colored self, Color c)"""
        return _IMP_display.Colored_set_color(self, c)

    def get_color(self):
        r"""get_color(Colored self) -> Color"""
        return _IMP_display.Colored_get_color(self)

    @staticmethod
    def get_color_keys():
        r"""get_color_keys() -> IMP::FloatKeys const &"""
        return _IMP_display.Colored_get_color_keys()

    def __init__(self, *args):
        r"""
        __init__(Colored self) -> Colored
        __init__(Colored self, Model m, ParticleIndex id) -> Colored
        __init__(Colored self, _ParticleAdaptor d) -> Colored
        """
        _IMP_display.Colored_swiginit(self, _IMP_display.new_Colored(*args))

    @staticmethod
    def get_is_setup(*args):
        r"""
        get_is_setup(Model m, ParticleIndex pi) -> bool
        get_is_setup(_ParticleAdaptor p) -> bool
        """
        return _IMP_display.Colored_get_is_setup(*args)

    def show(self, *args):
        r"""show(Colored self, _ostream out=std::cout)"""
        return _IMP_display.Colored_show(self, *args)

    @staticmethod
    def setup_particle(*args):
        r"""
        setup_particle(Model m, ParticleIndex pi, Color color) -> Colored
        setup_particle(_ParticleAdaptor decorator, Color color) -> Colored
        """
        return _IMP_display.Colored_setup_particle(*args)

    def add_attribute(self, *args):
        r"""
        add_attribute(Colored self, FloatKey k, IMP::Float v, bool opt)
        add_attribute(Colored self, FloatKey a0, IMP::Float a1)
        add_attribute(Colored self, IntKey a0, IMP::Int a1)
        add_attribute(Colored self, FloatsKey a0, IMP::Floats a1)
        add_attribute(Colored self, IntsKey a0, IMP::Ints a1)
        add_attribute(Colored self, StringKey a0, IMP::String a1)
        add_attribute(Colored self, ParticleIndexKey a0, Particle a1)
        add_attribute(Colored self, ObjectKey a0, Object a1)
        """
        return _IMP_display.Colored_add_attribute(self, *args)

    def get_value(self, *args):
        r"""
        get_value(Colored self, FloatKey a0) -> IMP::Float
        get_value(Colored self, IntKey a0) -> IMP::Int
        get_value(Colored self, FloatsKey a0) -> IMP::Floats
        get_value(Colored self, IntsKey a0) -> IMP::Ints
        get_value(Colored self, StringKey a0) -> IMP::String
        get_value(Colored self, ParticleIndexKey a0) -> Particle
        get_value(Colored self, ObjectKey a0) -> Object
        """
        return _IMP_display.Colored_get_value(self, *args)

    def set_value(self, *args):
        r"""
        set_value(Colored self, FloatKey a0, IMP::Float a1)
        set_value(Colored self, IntKey a0, IMP::Int a1)
        set_value(Colored self, FloatsKey a0, IMP::Floats a1)
        set_value(Colored self, IntsKey a0, IMP::Ints a1)
        set_value(Colored self, StringKey a0, IMP::String a1)
        set_value(Colored self, ParticleIndexKey a0, Particle a1)
        set_value(Colored self, ObjectKey a0, Object a1)
        """
        return _IMP_display.Colored_set_value(self, *args)

    def remove_attribute(self, *args):
        r"""
        remove_attribute(Colored self, FloatKey a0)
        remove_attribute(Colored self, IntKey a0)
        remove_attribute(Colored self, FloatsKey a0)
        remove_attribute(Colored self, IntsKey a0)
        remove_attribute(Colored self, StringKey a0)
        remove_attribute(Colored self, ParticleIndexKey a0)
        remove_attribute(Colored self, ObjectKey a0)
        """
        return _IMP_display.Colored_remove_attribute(self, *args)

    def has_attribute(self, *args):
        r"""
        has_attribute(Colored self, FloatKey a0) -> bool
        has_attribute(Colored self, IntKey a0) -> bool
        has_attribute(Colored self, FloatsKey a0) -> bool
        has_attribute(Colored self, IntsKey a0) -> bool
        has_attribute(Colored self, StringKey a0) -> bool
        has_attribute(Colored self, ParticleIndexKey a0) -> bool
        has_attribute(Colored self, ObjectKey a0) -> bool
        """
        return _IMP_display.Colored_has_attribute(self, *args)

    def get_derivative(self, a0):
        r"""get_derivative(Colored self, FloatKey a0) -> double"""
        return _IMP_display.Colored_get_derivative(self, a0)

    def get_name(self):
        r"""get_name(Colored self) -> std::string"""
        return _IMP_display.Colored_get_name(self)

    def clear_caches(self):
        r"""clear_caches(Colored self)"""
        return _IMP_display.Colored_clear_caches(self)

    def set_name(self, a0):
        r"""set_name(Colored self, std::string a0)"""
        return _IMP_display.Colored_set_name(self, a0)

    def set_check_level(self, a0):
        r"""set_check_level(Colored self, IMP::CheckLevel a0)"""
        return _IMP_display.Colored_set_check_level(self, a0)

    def add_to_derivative(self, a0, a1, a2):
        r"""add_to_derivative(Colored self, FloatKey a0, double a1, DerivativeAccumulator a2)"""
        return _IMP_display.Colored_add_to_derivative(self, a0, a1, a2)

    def set_is_optimized(self, a0, a1):
        r"""set_is_optimized(Colored self, FloatKey a0, bool a1)"""
        return _IMP_display.Colored_set_is_optimized(self, a0, a1)

    def get_is_optimized(self, a0):
        r"""get_is_optimized(Colored self, FloatKey a0) -> bool"""
        return _IMP_display.Colored_get_is_optimized(self, a0)

    def get_check_level(self):
        r"""get_check_level(Colored self) -> IMP::CheckLevel"""
        return _IMP_display.Colored_get_check_level(self)

    def __eq__(self, *args):
        r"""
        __eq__(Colored self, Colored o) -> bool
        __eq__(Colored self, Particle d) -> bool
        """
        return _IMP_display.Colored___eq__(self, *args)

    def __ne__(self, *args):
        r"""
        __ne__(Colored self, Colored o) -> bool
        __ne__(Colored self, Particle d) -> bool
        """
        return _IMP_display.Colored___ne__(self, *args)

    def __le__(self, *args):
        r"""
        __le__(Colored self, Colored o) -> bool
        __le__(Colored self, Particle d) -> bool
        """
        return _IMP_display.Colored___le__(self, *args)

    def __lt__(self, *args):
        r"""
        __lt__(Colored self, Colored o) -> bool
        __lt__(Colored self, Particle d) -> bool
        """
        return _IMP_display.Colored___lt__(self, *args)

    def __ge__(self, *args):
        r"""
        __ge__(Colored self, Colored o) -> bool
        __ge__(Colored self, Particle d) -> bool
        """
        return _IMP_display.Colored___ge__(self, *args)

    def __gt__(self, *args):
        r"""
        __gt__(Colored self, Colored o) -> bool
        __gt__(Colored self, Particle d) -> bool
        """
        return _IMP_display.Colored___gt__(self, *args)

    def __hash__(self):
        r"""__hash__(Colored self) -> std::size_t"""
        return _IMP_display.Colored___hash__(self)

    def __str__(self):
        r"""__str__(Colored self) -> std::string"""
        return _IMP_display.Colored___str__(self)

    def __repr__(self):
        r"""__repr__(Colored self) -> std::string"""
        return _IMP_display.Colored___repr__(self)
    __swig_destroy__ = _IMP_display.delete_Colored

# Register Colored in _IMP_display:
_IMP_display.Colored_swigregister(Colored)

def Colored_get_color_keys():
    r"""Colored_get_color_keys() -> IMP::FloatKeys const &"""
    return _IMP_display.Colored_get_color_keys()

def Colored_get_is_setup(*args):
    r"""
    Colored_get_is_setup(Model m, ParticleIndex pi) -> bool
    Colored_get_is_setup(_ParticleAdaptor p) -> bool
    """
    return _IMP_display.Colored_get_is_setup(*args)

def Colored_setup_particle(*args):
    r"""
    Colored_setup_particle(Model m, ParticleIndex pi, Color color) -> Colored
    Colored_setup_particle(_ParticleAdaptor decorator, Color color) -> Colored
    """
    return _IMP_display.Colored_setup_particle(*args)


def __lshift__(out, n):
    r"""__lshift__(_ostream out, Colored n) -> _ostream"""
    return _IMP_display.__lshift__(out, n)
class FilterGeometry(GeometryProcessor, Geometry):
    r"""Proxy of C++ IMP::display::FilterGeometry class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, p):
        r"""__init__(FilterGeometry self, Plane3D p) -> FilterGeometry"""
        _IMP_display.FilterGeometry_swiginit(self, _IMP_display.new_FilterGeometry(p))

    def add_geometry(self, *args):
        r"""
        add_geometry(FilterGeometry self, Geometry g)
        add_geometry(FilterGeometry self, IMP::display::Geometries const & g)
        """
        return _IMP_display.FilterGeometry_add_geometry(self, *args)

    def get_version_info(self):
        r"""get_version_info(FilterGeometry self) -> VersionInfo"""
        return _IMP_display.FilterGeometry_get_version_info(self)

    def __str__(self):
        r"""__str__(FilterGeometry self) -> std::string"""
        return _IMP_display.FilterGeometry___str__(self)

    def __repr__(self):
        r"""__repr__(FilterGeometry self) -> std::string"""
        return _IMP_display.FilterGeometry___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_FilterGeometry(o)


# Register FilterGeometry in _IMP_display:
_IMP_display.FilterGeometry_swigregister(FilterGeometry)

class WriteOptimizerState(IMP.OptimizerState):
    r"""Proxy of C++ IMP::display::WriteOptimizerState class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, m, w):
        r"""__init__(WriteOptimizerState self, Model m, WriterAdaptor w) -> WriteOptimizerState"""
        _IMP_display.WriteOptimizerState_swiginit(self, _IMP_display.new_WriteOptimizerState(m, w))

    def write(self, w):
        r"""write(WriteOptimizerState self, WriterAdaptor w)"""
        return _IMP_display.WriteOptimizerState_write(self, w)

    def remove_geometry(self, d):
        r"""remove_geometry(WriteOptimizerState self, Geometry d)"""
        return _IMP_display.WriteOptimizerState_remove_geometry(self, d)

    def remove_geometries(self, d):
        r"""remove_geometries(WriteOptimizerState self, IMP::display::Geometries const & d)"""
        return _IMP_display.WriteOptimizerState_remove_geometries(self, d)

    def set_geometries(self, ps):
        r"""set_geometries(WriteOptimizerState self, IMP::display::Geometries const & ps)"""
        return _IMP_display.WriteOptimizerState_set_geometries(self, ps)

    def set_geometries_order(self, objs):
        r"""set_geometries_order(WriteOptimizerState self, IMP::display::Geometries const & objs)"""
        return _IMP_display.WriteOptimizerState_set_geometries_order(self, objs)

    def add_geometry(self, obj):
        r"""add_geometry(WriteOptimizerState self, Geometry obj) -> unsigned int"""
        return _IMP_display.WriteOptimizerState_add_geometry(self, obj)

    def add_geometries(self, objs):
        r"""add_geometries(WriteOptimizerState self, IMP::display::Geometries const & objs)"""
        return _IMP_display.WriteOptimizerState_add_geometries(self, objs)

    def clear_geometries(self):
        r"""clear_geometries(WriteOptimizerState self)"""
        return _IMP_display.WriteOptimizerState_clear_geometries(self)

    def get_number_of_geometries(self):
        r"""get_number_of_geometries(WriteOptimizerState self) -> unsigned int"""
        return _IMP_display.WriteOptimizerState_get_number_of_geometries(self)

    def get_has_geometries(self):
        r"""get_has_geometries(WriteOptimizerState self) -> bool"""
        return _IMP_display.WriteOptimizerState_get_has_geometries(self)

    def get_geometry(self, i):
        r"""get_geometry(WriteOptimizerState self, unsigned int i) -> Geometry"""
        return _IMP_display.WriteOptimizerState_get_geometry(self, i)

    def get_geometries(self):
        r"""get_geometries(WriteOptimizerState self) -> IMP::display::Geometries"""
        return _IMP_display.WriteOptimizerState_get_geometries(self)

    def reserve_geometries(self, sz):
        r"""reserve_geometries(WriteOptimizerState self, unsigned int sz)"""
        return _IMP_display.WriteOptimizerState_reserve_geometries(self, sz)

    def get_version_info(self):
        r"""get_version_info(WriteOptimizerState self) -> VersionInfo"""
        return _IMP_display.WriteOptimizerState_get_version_info(self)

    def __str__(self):
        r"""__str__(WriteOptimizerState self) -> std::string"""
        return _IMP_display.WriteOptimizerState___str__(self)

    def __repr__(self):
        r"""__repr__(WriteOptimizerState self) -> std::string"""
        return _IMP_display.WriteOptimizerState___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_WriteOptimizerState(o)


# Register WriteOptimizerState in _IMP_display:
_IMP_display.WriteOptimizerState_swigregister(WriteOptimizerState)

class SingletonGeometry(Geometry):
    r"""Proxy of C++ IMP::display::SingletonGeometry class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, p):
        r"""__init__(SingletonGeometry self, Particle p) -> SingletonGeometry"""
        _IMP_display.SingletonGeometry_swiginit(self, _IMP_display.new_SingletonGeometry(p))

    def get_particle(self):
        r"""get_particle(SingletonGeometry self) -> Particle"""
        return _IMP_display.SingletonGeometry_get_particle(self)
    __swig_destroy__ = _IMP_display.delete_SingletonGeometry

    def __str__(self):
        r"""__str__(SingletonGeometry self) -> std::string"""
        return _IMP_display.SingletonGeometry___str__(self)

    def __repr__(self):
        r"""__repr__(SingletonGeometry self) -> std::string"""
        return _IMP_display.SingletonGeometry___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_SingletonGeometry(o)


# Register SingletonGeometry in _IMP_display:
_IMP_display.SingletonGeometry_swigregister(SingletonGeometry)

class SingletonsGeometry(Geometry):
    r"""Proxy of C++ IMP::display::SingletonsGeometry class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args):
        r"""
        __init__(SingletonsGeometry self, _SingletonContainerAdaptor pc, Color c) -> SingletonsGeometry
        __init__(SingletonsGeometry self, _SingletonContainerAdaptor pc) -> SingletonsGeometry
        """
        _IMP_display.SingletonsGeometry_swiginit(self, _IMP_display.new_SingletonsGeometry(*args))

    def get_container(self):
        r"""get_container(SingletonsGeometry self) -> SingletonContainer"""
        return _IMP_display.SingletonsGeometry_get_container(self)
    __swig_destroy__ = _IMP_display.delete_SingletonsGeometry

    def __str__(self):
        r"""__str__(SingletonsGeometry self) -> std::string"""
        return _IMP_display.SingletonsGeometry___str__(self)

    def __repr__(self):
        r"""__repr__(SingletonsGeometry self) -> std::string"""
        return _IMP_display.SingletonsGeometry___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_SingletonsGeometry(o)


# Register SingletonsGeometry in _IMP_display:
_IMP_display.SingletonsGeometry_swigregister(SingletonsGeometry)

class PairGeometry(Geometry):
    r"""Proxy of C++ IMP::display::PairGeometry class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, pp):
        r"""__init__(PairGeometry self, IMP::ParticlePair const & pp) -> PairGeometry"""
        _IMP_display.PairGeometry_swiginit(self, _IMP_display.new_PairGeometry(pp))

    def get_particle_pair(self):
        r"""get_particle_pair(PairGeometry self) -> IMP::ParticlePair"""
        return _IMP_display.PairGeometry_get_particle_pair(self)
    __swig_destroy__ = _IMP_display.delete_PairGeometry

    def __str__(self):
        r"""__str__(PairGeometry self) -> std::string"""
        return _IMP_display.PairGeometry___str__(self)

    def __repr__(self):
        r"""__repr__(PairGeometry self) -> std::string"""
        return _IMP_display.PairGeometry___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_PairGeometry(o)


# Register PairGeometry in _IMP_display:
_IMP_display.PairGeometry_swigregister(PairGeometry)

class PairsGeometry(Geometry):
    r"""Proxy of C++ IMP::display::PairsGeometry class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args):
        r"""
        __init__(PairsGeometry self, PairContainer pc, Color c) -> PairsGeometry
        __init__(PairsGeometry self, PairContainer pc) -> PairsGeometry
        """
        _IMP_display.PairsGeometry_swiginit(self, _IMP_display.new_PairsGeometry(*args))

    def get_container(self):
        r"""get_container(PairsGeometry self) -> PairContainer"""
        return _IMP_display.PairsGeometry_get_container(self)
    __swig_destroy__ = _IMP_display.delete_PairsGeometry

    def __str__(self):
        r"""__str__(PairsGeometry self) -> std::string"""
        return _IMP_display.PairsGeometry___str__(self)

    def __repr__(self):
        r"""__repr__(PairsGeometry self) -> std::string"""
        return _IMP_display.PairsGeometry___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_PairsGeometry(o)


# Register PairsGeometry in _IMP_display:
_IMP_display.PairsGeometry_swigregister(PairsGeometry)

class PymolWriter(TextWriter):
    r"""Proxy of C++ IMP::display::PymolWriter class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args):
        r"""
        __init__(PymolWriter self, TextOutput of) -> PymolWriter
        __init__(PymolWriter self, std::string name) -> PymolWriter
        __init__(PymolWriter self, char const * name) -> PymolWriter
        """
        _IMP_display.PymolWriter_swiginit(self, _IMP_display.new_PymolWriter(*args))

    def get_version_info(self):
        r"""get_version_info(PymolWriter self) -> VersionInfo"""
        return _IMP_display.PymolWriter_get_version_info(self)

    def __str__(self):
        r"""__str__(PymolWriter self) -> std::string"""
        return _IMP_display.PymolWriter___str__(self)

    def __repr__(self):
        r"""__repr__(PymolWriter self) -> std::string"""
        return _IMP_display.PymolWriter___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_PymolWriter(o)


# Register PymolWriter in _IMP_display:
_IMP_display.PymolWriter_swigregister(PymolWriter)

class RestraintGeometry(Geometry):
    r"""Proxy of C++ IMP::display::RestraintGeometry class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, p):
        r"""__init__(RestraintGeometry self, Restraint p) -> RestraintGeometry"""
        _IMP_display.RestraintGeometry_swiginit(self, _IMP_display.new_RestraintGeometry(p))

    def get_restraint(self):
        r"""get_restraint(RestraintGeometry self) -> Restraint"""
        return _IMP_display.RestraintGeometry_get_restraint(self)

    def get_version_info(self):
        r"""get_version_info(RestraintGeometry self) -> VersionInfo"""
        return _IMP_display.RestraintGeometry_get_version_info(self)

    def __str__(self):
        r"""__str__(RestraintGeometry self) -> std::string"""
        return _IMP_display.RestraintGeometry___str__(self)

    def __repr__(self):
        r"""__repr__(RestraintGeometry self) -> std::string"""
        return _IMP_display.RestraintGeometry___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_RestraintGeometry(o)


# Register RestraintGeometry in _IMP_display:
_IMP_display.RestraintGeometry_swigregister(RestraintGeometry)

class RestraintSetGeometry(Geometry):
    r"""Proxy of C++ IMP::display::RestraintSetGeometry class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, p):
        r"""__init__(RestraintSetGeometry self, RestraintSet p) -> RestraintSetGeometry"""
        _IMP_display.RestraintSetGeometry_swiginit(self, _IMP_display.new_RestraintSetGeometry(p))

    def get_restraint_set(self):
        r"""get_restraint_set(RestraintSetGeometry self) -> RestraintSet"""
        return _IMP_display.RestraintSetGeometry_get_restraint_set(self)

    def get_version_info(self):
        r"""get_version_info(RestraintSetGeometry self) -> VersionInfo"""
        return _IMP_display.RestraintSetGeometry_get_version_info(self)

    def __str__(self):
        r"""__str__(RestraintSetGeometry self) -> std::string"""
        return _IMP_display.RestraintSetGeometry___str__(self)

    def __repr__(self):
        r"""__repr__(RestraintSetGeometry self) -> std::string"""
        return _IMP_display.RestraintSetGeometry___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_RestraintSetGeometry(o)


# Register RestraintSetGeometry in _IMP_display:
_IMP_display.RestraintSetGeometry_swigregister(RestraintSetGeometry)


from . import _pivy
PivyWriter= _pivy.get_writer(Writer)
_object_types.append('PivyWriter')


def get_module_version():
    r"""get_module_version() -> std::string const"""
    return _IMP_display.get_module_version()

def get_example_path(fname):
    r"""get_example_path(std::string fname) -> std::string"""
    return _IMP_display.get_example_path(fname)

def get_data_path(fname):
    r"""get_data_path(std::string fname) -> std::string"""
    return _IMP_display.get_data_path(fname)

from . import _version_check
_version_check.check_version(get_module_version())
__version__ = get_module_version()






#! /usr/bin/env python


from __future__ import print_function
import argparse

import IMP
import IMP.rmf
import RMF
import difflib


parser = argparse.ArgumentParser(
    description='Process output data file saved as rmfs. It has two modality: print selected fields for all lines or print a particular line where a filed has a given value.')
parser.add_argument(
    '-f',
    action="store",
    dest="filename",
    help="file name to process")
parser.add_argument(
    '-s',
    dest="fields",
    nargs="+",
    help="Specify all fields to be printed. Multiple flags will append a list of fields to be printed")
parser.add_argument(
    '-p',
    action="store_true",
    dest="print_fields",
    default=False,
    help="print the fields contained in the file")
parser.add_argument(
    '-n',
    action="store",
    dest="nframe",
    default=None,
    help="Print frame number n")

result = parser.parse_args()


# open the file
if not result.filename is None:
    try:
        #let's see if that is an rmf file
        rh = RMF.open_rmf_file_read_only(result.filename)
        cat=rh.get_category('stat')
        rmf_klist=rh.get_keys(cat)
        rmf_names_keys=dict([(rh.get_name(k),k) for k in rmf_klist])
        klist=rmf_names_keys.keys()
        del rh
    except IOError:
        raise IOError("Not an RMF file")
else:
    raise ValueError("No file name provided. Use -h for help")


# print the keys
if result.print_fields:
    for key in klist:
        print(key)


# the field string matching is by default strict, i.e., the input string
# must be the same as the one in the file
match_strictness = 1.0

# print the queried fields
if not result.fields is None:
    field_list = []
    # check whether the fields exist and convert them to best maching existing
    # field names
    for field in result.fields:
        found_entries = difflib.get_close_matches(
            field,
            klist,
            1,
            match_strictness)
        if len(found_entries) == 0:
            raise ValueError("field " + field + " non found")
        else:
            field_list.append(found_entries[0])

    # print comment line
    s0 = ' '.join(["%20s" % (field) for field in field_list])
    print("# " + s0)

    # print fields values
    rh = RMF.open_rmf_file_read_only(result.filename)

    for frame_number in range(rh.get_number_of_frames()):
        IMP.rmf.load_frame(rh, RMF.FrameID(frame_number))
        for name in klist:
            s0=' '.join(["%20s" % (str(rh.get_root_node().get_value(rmf_names_keys[name])))
                          for name in field_list])

        if not result.nframe:
            print("> " + s0)
        else:
            print(str(frame_number)+ " > " + s0)
    del rh

# print given frame number
if not result.nframe is None:
    # print comment line
    # print fields values
    rh = RMF.open_rmf_file_read_only(result.filename)
    IMP.rmf.load_frame(rh, RMF.FrameID(int(result.nframe)))
    for k in klist:
        s0=str(k)+" "+str(rh.get_root_node().get_value(rmf_names_keys[k]))
        print(s0)
    del rh



## \example core/connectivity_restraint.py
# This example shows how to use the ConnectivityRestraint to ensure that
# all the particles end up in a connected conformation following the
# optimization. One should also check out the
# IMP::atom::create_connectivity_restraint() helper functions.

import IMP
import IMP.core
import IMP.container
import IMP.algebra
import IMP.atom
import sys

IMP.setup_from_argv(sys.argv, "Connectivity restraint")

m = IMP.Model()

# Put the parent particles for each molecule
hs = []

# create the molecules, with 5 particles for each of 10 molecules
for i in range(0, 10):
    pr = IMP.Particle(m)
    pr.set_name("root " + str(i))
    d = IMP.atom.Hierarchy.setup_particle(pr)
    for j in range(0, 5):
        p = IMP.Particle(m)
        p.set_name("fragment " + str(i) + " " + str(j))
        cd = IMP.atom.Fragment.setup_particle(p)
        d.add_child(cd)
        xd = IMP.core.XYZR.setup_particle(
            p, IMP.algebra.Sphere3D(IMP.algebra.Vector3D(3 * i, j, 0), 1))
    hs.append(pr)


ps = IMP.core.SphereDistancePairScore(IMP.core.HarmonicUpperBound(0, 1))
cps = IMP.core.ChildrenRefiner(IMP.atom.Hierarchy.get_traits())

# score based on the one closest particle from each set of balls
lrps = IMP.core.KClosePairsPairScore(ps, cps, 1)
# connect all 10 molecules together
lsc = IMP.container.ListSingletonContainer(m, hs)
cr = IMP.core.ConnectivityRestraint(lrps, lsc)

cr.evaluate(False)



# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

# This wrapper is part of IMP,
# Copyright 2007-2021 IMP Inventors. All rights reserved.

from __future__ import print_function, division, absolute_import
from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

import _IMP_kmeans

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

class IMP_KMEANS_SwigPyIterator(object):
    r"""Proxy of C++ swig::IMP_KMEANS_SwigPyIterator class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    __swig_destroy__ = _IMP_kmeans.delete_IMP_KMEANS_SwigPyIterator

    def value(self):
        r"""value(IMP_KMEANS_SwigPyIterator self) -> PyObject *"""
        return _IMP_kmeans.IMP_KMEANS_SwigPyIterator_value(self)

    def incr(self, n=1):
        r"""incr(IMP_KMEANS_SwigPyIterator self, size_t n=1) -> IMP_KMEANS_SwigPyIterator"""
        return _IMP_kmeans.IMP_KMEANS_SwigPyIterator_incr(self, n)

    def decr(self, n=1):
        r"""decr(IMP_KMEANS_SwigPyIterator self, size_t n=1) -> IMP_KMEANS_SwigPyIterator"""
        return _IMP_kmeans.IMP_KMEANS_SwigPyIterator_decr(self, n)

    def distance(self, x):
        r"""distance(IMP_KMEANS_SwigPyIterator self, IMP_KMEANS_SwigPyIterator x) -> ptrdiff_t"""
        return _IMP_kmeans.IMP_KMEANS_SwigPyIterator_distance(self, x)

    def equal(self, x):
        r"""equal(IMP_KMEANS_SwigPyIterator self, IMP_KMEANS_SwigPyIterator x) -> bool"""
        return _IMP_kmeans.IMP_KMEANS_SwigPyIterator_equal(self, x)

    def copy(self):
        r"""copy(IMP_KMEANS_SwigPyIterator self) -> IMP_KMEANS_SwigPyIterator"""
        return _IMP_kmeans.IMP_KMEANS_SwigPyIterator_copy(self)

    def next(self):
        r"""next(IMP_KMEANS_SwigPyIterator self) -> PyObject *"""
        return _IMP_kmeans.IMP_KMEANS_SwigPyIterator_next(self)

    def __next__(self):
        r"""__next__(IMP_KMEANS_SwigPyIterator self) -> PyObject *"""
        return _IMP_kmeans.IMP_KMEANS_SwigPyIterator___next__(self)

    def previous(self):
        r"""previous(IMP_KMEANS_SwigPyIterator self) -> PyObject *"""
        return _IMP_kmeans.IMP_KMEANS_SwigPyIterator_previous(self)

    def advance(self, n):
        r"""advance(IMP_KMEANS_SwigPyIterator self, ptrdiff_t n) -> IMP_KMEANS_SwigPyIterator"""
        return _IMP_kmeans.IMP_KMEANS_SwigPyIterator_advance(self, n)

    def __eq__(self, x):
        r"""__eq__(IMP_KMEANS_SwigPyIterator self, IMP_KMEANS_SwigPyIterator x) -> bool"""
        return _IMP_kmeans.IMP_KMEANS_SwigPyIterator___eq__(self, x)

    def __ne__(self, x):
        r"""__ne__(IMP_KMEANS_SwigPyIterator self, IMP_KMEANS_SwigPyIterator x) -> bool"""
        return _IMP_kmeans.IMP_KMEANS_SwigPyIterator___ne__(self, x)

    def __iadd__(self, n):
        r"""__iadd__(IMP_KMEANS_SwigPyIterator self, ptrdiff_t n) -> IMP_KMEANS_SwigPyIterator"""
        return _IMP_kmeans.IMP_KMEANS_SwigPyIterator___iadd__(self, n)

    def __isub__(self, n):
        r"""__isub__(IMP_KMEANS_SwigPyIterator self, ptrdiff_t n) -> IMP_KMEANS_SwigPyIterator"""
        return _IMP_kmeans.IMP_KMEANS_SwigPyIterator___isub__(self, n)

    def __add__(self, n):
        r"""__add__(IMP_KMEANS_SwigPyIterator self, ptrdiff_t n) -> IMP_KMEANS_SwigPyIterator"""
        return _IMP_kmeans.IMP_KMEANS_SwigPyIterator___add__(self, n)

    def __sub__(self, *args):
        r"""
        __sub__(IMP_KMEANS_SwigPyIterator self, ptrdiff_t n) -> IMP_KMEANS_SwigPyIterator
        __sub__(IMP_KMEANS_SwigPyIterator self, IMP_KMEANS_SwigPyIterator x) -> ptrdiff_t
        """
        return _IMP_kmeans.IMP_KMEANS_SwigPyIterator___sub__(self, *args)
    def __iter__(self):
        return self

# Register IMP_KMEANS_SwigPyIterator in _IMP_kmeans:
_IMP_kmeans.IMP_KMEANS_SwigPyIterator_swigregister(IMP_KMEANS_SwigPyIterator)


_value_types=[]
_object_types=[]
_raii_types=[]
_plural_types=[]

IMP_DEBUG = _IMP_kmeans.IMP_DEBUG

IMP_RELEASE = _IMP_kmeans.IMP_RELEASE

IMP_SILENT = _IMP_kmeans.IMP_SILENT

IMP_PROGRESS = _IMP_kmeans.IMP_PROGRESS

IMP_TERSE = _IMP_kmeans.IMP_TERSE

IMP_VERBOSE = _IMP_kmeans.IMP_VERBOSE

IMP_MEMORY = _IMP_kmeans.IMP_MEMORY

IMP_NONE = _IMP_kmeans.IMP_NONE

IMP_USAGE = _IMP_kmeans.IMP_USAGE

IMP_INTERNAL = _IMP_kmeans.IMP_INTERNAL

IMP_KERNEL_HAS_LOG4CXX = _IMP_kmeans.IMP_KERNEL_HAS_LOG4CXX

IMP_COMPILER_HAS_AUTO = _IMP_kmeans.IMP_COMPILER_HAS_AUTO

IMP_COMPILER_HAS_DEBUG_VECTOR = _IMP_kmeans.IMP_COMPILER_HAS_DEBUG_VECTOR

IMP_COMPILER_HAS_UNIQUE_PTR = _IMP_kmeans.IMP_COMPILER_HAS_UNIQUE_PTR

IMP_KERNEL_HAS_BOOST_RANDOM = _IMP_kmeans.IMP_KERNEL_HAS_BOOST_RANDOM

IMP_KERNEL_HAS_NUMPY = _IMP_kmeans.IMP_KERNEL_HAS_NUMPY

IMP_KERNEL_HAS_GPERFTOOLS = _IMP_kmeans.IMP_KERNEL_HAS_GPERFTOOLS

IMP_KERNEL_HAS_TCMALLOC_HEAPCHECKER = _IMP_kmeans.IMP_KERNEL_HAS_TCMALLOC_HEAPCHECKER

IMP_KERNEL_HAS_TCMALLOC_HEAPPROFILER = _IMP_kmeans.IMP_KERNEL_HAS_TCMALLOC_HEAPPROFILER

IMPKERNEL_SHOW_WARNINGS = _IMP_kmeans.IMPKERNEL_SHOW_WARNINGS


import sys
class _DirectorObjects(object):
    """@internal Simple class to keep references to director objects
       to prevent premature deletion."""
    def __init__(self):
        self._objects = []
    def register(self, obj):
        """Take a reference to a director object; will only work for
           refcounted C++ classes"""
        if hasattr(obj, 'get_ref_count'):
            self._objects.append(obj)
    def cleanup(self):
        """Only drop our reference and allow cleanup by Python if no other
           Python references exist (we hold 3 references: one in self._objects,
           one in x, and one in the argument list for getrefcount) *and* no
           other C++ references exist (the Python object always holds one)"""
        objs = [x for x in self._objects if sys.getrefcount(x) > 3 \
                                         or x.get_ref_count() > 1]
# Do in two steps so the references are kept until the end of the
# function (deleting references may trigger a fresh call to this method)
        self._objects = objs
    def get_object_count(self):
        """Get number of director objects (useful for testing only)"""
        return len(self._objects)
_director_objects = _DirectorObjects()

class _ostream(object):
    r"""Proxy of C++ std::ostream class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    def write(self, osa_buf):
        r"""write(_ostream self, char const * osa_buf)"""
        return _IMP_kmeans._ostream_write(self, osa_buf)

# Register _ostream in _IMP_kmeans:
_IMP_kmeans._ostream_swigregister(_ostream)

IMP_COMPILER_HAS_OVERRIDE = _IMP_kmeans.IMP_COMPILER_HAS_OVERRIDE

IMP_COMPILER_HAS_FINAL = _IMP_kmeans.IMP_COMPILER_HAS_FINAL

IMP_HAS_NOEXCEPT = _IMP_kmeans.IMP_HAS_NOEXCEPT

IMP_C_OPEN_BINARY = _IMP_kmeans.IMP_C_OPEN_BINARY

import IMP
IMP_KMEANS_HAS_BOOST_FILESYSTEM = _IMP_kmeans.IMP_KMEANS_HAS_BOOST_FILESYSTEM

IMP_KMEANS_HAS_BOOST_PROGRAMOPTIONS = _IMP_kmeans.IMP_KMEANS_HAS_BOOST_PROGRAMOPTIONS

IMP_KMEANS_HAS_BOOST_RANDOM = _IMP_kmeans.IMP_KMEANS_HAS_BOOST_RANDOM

IMP_KMEANS_HAS_BOOST_SYSTEM = _IMP_kmeans.IMP_KMEANS_HAS_BOOST_SYSTEM

IMP_KMEANS_HAS_NUMPY = _IMP_kmeans.IMP_KMEANS_HAS_NUMPY

IMPKMEANS_SHOW_WARNINGS = _IMP_kmeans.IMPKMEANS_SHOW_WARNINGS


_object_types.append("KMeans")


def _object_cast_to_KMeans(o):
    r"""_object_cast_to_KMeans(Object o) -> KMeans"""
    return _IMP_kmeans._object_cast_to_KMeans(o)
KM_LLOYDS = _IMP_kmeans.KM_LLOYDS

KM_LOCAL_SWAP = _IMP_kmeans.KM_LOCAL_SWAP

KM_LOCAL_EZ_HYBRID = _IMP_kmeans.KM_LOCAL_EZ_HYBRID

KM_HYBRID = _IMP_kmeans.KM_HYBRID

class KMeans(IMP.Object):
    r"""Proxy of C++ IMP::kmeans::KMeans class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args):
        r"""
        __init__(KMeans self, std::string const & fname_data, int dim, unsigned int max_nPts) -> KMeans
        __init__(KMeans self) -> KMeans
        """
        _IMP_kmeans.KMeans_swiginit(self, _IMP_kmeans.new_KMeans(*args))

    def get_version_info(self):
        r"""get_version_info(KMeans self) -> VersionInfo"""
        return _IMP_kmeans.KMeans_get_version_info(self)

    def execute(self, *args):
        r"""execute(KMeans self, unsigned int k, IMP::kmeans::KM_ALG_TYPE alg_type=KM_LLOYDS, int stages=100)"""
        return _IMP_kmeans.KMeans_execute(self, *args)

    def add_data_pt(self, p):
        r"""add_data_pt(KMeans self, IMP::Floats const & p)"""
        return _IMP_kmeans.KMeans_add_data_pt(self, p)

    def clear_data(self):
        r"""clear_data(KMeans self)"""
        return _IMP_kmeans.KMeans_clear_data(self)

    def get_data_point(self, i):
        r"""get_data_point(KMeans self, unsigned int i) -> IMP::Floats const &"""
        return _IMP_kmeans.KMeans_get_data_point(self, i)

    def get_number_of_data_points(self):
        r"""get_number_of_data_points(KMeans self) -> unsigned int"""
        return _IMP_kmeans.KMeans_get_number_of_data_points(self)

    def print_centers(self, *args):
        r"""print_centers(KMeans self, IMP::LogLevel ll=PROGRESS)"""
        return _IMP_kmeans.KMeans_print_centers(self, *args)

    def get_center(self, i):
        r"""get_center(KMeans self, unsigned int i) -> IMP::Floats"""
        return _IMP_kmeans.KMeans_get_center(self, i)

    def get_assignments(self):
        r"""get_assignments(KMeans self) -> IMP::Ints"""
        return _IMP_kmeans.KMeans_get_assignments(self)

    def get_squared_distance_to_centers(self):
        r"""get_squared_distance_to_centers(KMeans self) -> IMP::Floats"""
        return _IMP_kmeans.KMeans_get_squared_distance_to_centers(self)

    def get_number_of_centers(self):
        r"""get_number_of_centers(KMeans self) -> unsigned int"""
        return _IMP_kmeans.KMeans_get_number_of_centers(self)

    def __str__(self):
        r"""__str__(KMeans self) -> std::string"""
        return _IMP_kmeans.KMeans___str__(self)

    def __repr__(self):
        r"""__repr__(KMeans self) -> std::string"""
        return _IMP_kmeans.KMeans___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_KMeans(o)


# Register KMeans in _IMP_kmeans:
_IMP_kmeans.KMeans_swigregister(KMeans)


def get_module_version():
    r"""get_module_version() -> std::string const"""
    return _IMP_kmeans.get_module_version()

def get_example_path(fname):
    r"""get_example_path(std::string fname) -> std::string"""
    return _IMP_kmeans.get_example_path(fname)

def get_data_path(fname):
    r"""get_data_path(std::string fname) -> std::string"""
    return _IMP_kmeans.get_data_path(fname)

from . import _version_check
_version_check.check_version(get_module_version())
__version__ = get_module_version()






import IMP
import subprocess
import random
import IMP.domino
import IMP.core
import IMP.rmf
import RMF
import time
import IMP.algebra
import types
import re
import sys
import operator
import os
import peptideDocker
import atomicDomino

parameterFileName = sys.argv[1]

startTime = time.time()

p = peptideDocker.PeptideDocker(parameterFileName)

p.createModel()

p.loadHelpers()

p.initDof()

p.addForceFieldRestraints()
p.setInitialPositions()
p.addClosePairNonBondedRestraints()

d = atomicDomino.AtomicDomino(p.getModel(), p.getProtein(), parameterFileName)

d.loadDominoHelpers()

d.createSubsets()
d.writeVisualization()
p.logTime("Created subsets")
p.addCompleteNonBondedRestraints()

p.logTime("Added complete nonbonded restraints")


# p.addExcludedVolume()
p.logTime("Setup")

p.runMolecularDynamics()

p.logTime("Run MD")

p.runAllCg()

p.logTime("Run CG")

d.createGrid()

d.discretizeNativeProtein()

particleNameList = d.getDominoParticleNames()

flexibleAtoms = p.getFlexibleAtoms()

d.readMdTrajectory(particleNameList, flexibleAtoms)

d.readCgTrajectories(particleNameList, flexibleAtoms)

p.logTime("Read Trajectory")

d.createParticleStatesTable()

p.logTime("create particle states table")

d.createAllSubsetAssignments()

p.logTime("Create Leaf assignments")

d.createSampler()

d.runDomino()

p.logTime("Ran Domino")

p.writeOutput()

p.outputTimes()

d.writeOutput(flexibleAtoms, startTime)



## \example kernel/write_a_restraint.py
# While we do not recommend doing serious work using restraints written in Python, it is often useful when prototyping or testing code. Copy this example and modify as needed.
#

from __future__ import print_function
import IMP
import sys

IMP.setup_from_argv(sys.argv, "Restraint example")

# a restraint which checks if particles are sorted in
# increasing order on k.


class MyRestraint(IMP.Restraint):
    # take the list of particles and the key to use

    def __init__(self, m, ps, k):
        IMP.Restraint.__init__(self, m, "MyRestraint %1%")
        self.ps = ps
        self.k = k

    def unprotected_evaluate(self, da):
        score = 0
        for i in range(1, len(self.ps)):
            p0 = self.ps[i - 1]
            p1 = self.ps[i]
            if p0.get_value(k) > p1.get_value(k):
                diff = (p0.get_value(k) - p1.get_value(k))
                score = score + diff
                p0.add_to_derivative(k, -1, da)
                p1.add_to_derivative(k, 1, da)
            else:
                if IMP.get_log_level() >= IMP.TERSE:
                    print(p0.get_name(), "and", p1.get_name(), " are ok")
        return score

    def do_get_inputs(self):
        return self.ps

# some code to create and evaluate it
k = IMP.FloatKey("a key")
m = IMP.Model()
ps = []
for i in range(0, 10):
    p = IMP.Particle(m)
    p.add_attribute(k, i)
    ps.append(p)
r = MyRestraint(m, ps, k)
# IMP.set_log_level(IMP.TERSE)
print(r.evaluate(True))



## \example em2d/optimize_em2d_with_montecarlo.py
# Example of optimizing an EM2D restraint using Monte Carlo.
#

from __future__ import print_function
import IMP
import IMP.core
import IMP.atom
import IMP.em2d
import IMP.algebra
import IMP.container
import random
import sys

IMP.setup_from_argv(sys.argv, "optimize EM2D with MonteCarlo")


# An Optimizer score to get the values of the statistics after a given set
# of evaluations
class WriteStatisticsOptimizerScore(IMP.OptimizerState):

    def __init__(self, m, restraints):
        IMP.OptimizerState.__init__(self, m, "WriteStats")
        self.count = 0
        self.restraints = restraints

    def update(self):
        if (self.count != 10):
            self.count += 1
            return
        else:
            self.count = 0
        o = self.get_optimizer()
        m = o.get_model()
        for r in self.restraints:
            print(r.get_name(), r.get_last_score())
        # for i in range(0,m.get_number_of_restraints()):
        #    r=m.get_restraint(i)
        #    print "restraint",r.get_name(),"value",r.evaluate(False)


# Get model from PDB file
IMP.set_log_level(IMP.TERSE)
m = IMP.Model()
prot = IMP.atom.read_pdb(
    IMP.em2d.get_example_path("1z5s.pdb"), m, IMP.atom.ATOMPDBSelector())
IMP.atom.add_radii(prot)

# get the chains
chains = IMP.atom.get_by_type(prot, IMP.atom.CHAIN_TYPE)
print("there are", len(chains), "chains in 1z5s.pdb")

# set the chains as rigid bodies
native_chain_centers = []
rigid_bodies = []
for c in chains:
    atoms = IMP.core.get_leaves(c)
    rbd = IMP.core.RigidBody.setup_particle(c, atoms)
    rbd.set_coordinates_are_optimized(True)
    rigid_bodies.append(rbd)
    print("chain has", rbd.get_number_of_members(), \
        "atoms", "coordinates: ", rbd.get_coordinates())
    native_chain_centers.append(rbd.get_coordinates())

bb = IMP.algebra.BoundingBox3D(IMP.algebra.Vector3D(-25, -40, -60),
                               IMP.algebra.Vector3D(25, 40, 60))
# rotate and translate the chains
for rbd in rigid_bodies:
    # rotation
    rotation = IMP.algebra.get_random_rotation_3d()
    transformation1 = IMP.algebra.get_rotation_about_point(
        rbd.get_coordinates(), rotation)
    # translation
    transformation2 = IMP.algebra.Transformation3D(
        IMP.algebra.get_random_vector_in(bb))
    # Apply
    final_transformation = IMP.algebra.compose(
        transformation1, transformation2)
    IMP.core.transform(rbd, final_transformation)
print("Writing transformed assembly")
IMP.atom.write_pdb(prot, "1z5s-transformed.pdb")

# set distance restraints measusring some distances between rigid bodies
# for the solution.
d01 = IMP.algebra.get_distance(
    native_chain_centers[0], native_chain_centers[1])
r01 = IMP.core.DistanceRestraint(m,
    IMP.core.Harmonic(d01, 1), chains[0], chains[1])
r01.set_name("distance 0-1")
d12 = IMP.algebra.get_distance(
    native_chain_centers[1], native_chain_centers[2])
r12 = IMP.core.DistanceRestraint(m,
    IMP.core.Harmonic(d12, 1), chains[1], chains[2])
r12.set_name("distance 1-2")
d23 = IMP.algebra.get_distance(
    native_chain_centers[2], native_chain_centers[3])
r23 = IMP.core.DistanceRestraint(m,
    IMP.core.Harmonic(d23, 1), chains[2], chains[3])
r23.set_name("distance 2-3")
d30 = IMP.algebra.get_distance(
    native_chain_centers[3], native_chain_centers[0])
r30 = IMP.core.DistanceRestraint(m,
    IMP.core.Harmonic(d30, 1), chains[3], chains[0])
r30.set_name("distance 3-0")
print("Distances in the solution: d01 =", \
    d01, "d12 =", d12, "d23 =", d23, "d30 =", d30)

# set em2D restraint
srw = IMP.em2d.SpiderImageReaderWriter()
selection_file = IMP.em2d.get_example_path("all-1z5s-projections.sel")
images_to_read_names = [IMP.get_relative_path(selection_file, x) for x in
                        IMP.em2d.read_selection_file(selection_file)]
em_images = IMP.em2d.read_images(images_to_read_names, srw)
print(len(em_images), "images read")

em2d_restraint = IMP.em2d.Em2DRestraint(m)
apix = 1.5  # sampling rate of the available EM images
# resolution at which you want to generate the projections of the model
# In principle you want "perfect" projections, so use the highest resolution
resolution = 1
# Number of projections to use for the initial registration
#  (coarse registration) to estimate the registration parameters
n_projections = 20
params = IMP.em2d.Em2DRestraintParameters(apix, resolution, n_projections)

# This method (recommended) uses preprocessing of the images and projections
# to speed-up the registration
params.coarse_registration_method = IMP.em2d.ALIGN2D_PREPROCESSING
# use true if you want to save the projections from the model that best
# match the Em images
params.save_match_images = False

score_function = IMP.em2d.EM2DScore()
em2d_restraint = IMP.em2d.Em2DRestraint(m)
em2d_restraint.setup(score_function, params)
em2d_restraint.set_images(em_images)
em2d_restraint.set_name("em2d restraint")
container = IMP.container.ListSingletonContainer(m, IMP.core.get_leaves(prot))
em2d_restraint.set_particles(container)
em2d_restraints_set = IMP.RestraintSet(m)

# The next two lines are commented, because the optimization of the example
# is expensive. To run the full example, uncomment them (It can take a few
# hours).
# em2d_restraints_set.add_restraint(em2d_restraint)
# em2d_restraints_set.set_weight(1000) # weight for the em2D restraint

# Create scoring function using all restraints
all_restraints = [r01, r12, r23, r30, em2d_restraints_set]
sf = IMP.core.RestraintsScoringFunction(all_restraints)

# MONTECARLO OPTIMIZATION
s = IMP.core.MonteCarlo(m)
s.set_scoring_function(sf)
# Add movers for the rigid bodies
movers = []
for rbd in rigid_bodies:
    movers.append(IMP.core.RigidBodyMover(m, rbd, 5, 2))
s.add_movers(movers)
print("MonteCarlo sampler has", s.get_number_of_movers(), "movers")
# Add an optimizer state to save intermediate configurations of the hierarchy
o_state = IMP.atom.WritePDBOptimizerState(chains, "intermediate-step-%1%.pdb")
o_state.set_period(10)
s.add_optimizer_state(o_state)

ostate2 = WriteStatisticsOptimizerScore(m, all_restraints)
s.add_optimizer_state(ostate2)

# Perform optimization
temperatures = [200, 100, 60, 40, 20, 5]
# 200 steps recommended for accurate optimization; a smaller number is used
# here for demonstration purposes
optimization_steps = 10
for T in temperatures:
    s.set_kt(T)
    s.optimize(optimization_steps)
IMP.atom.write_pdb(prot, "solution.pdb")


# Check that the optimization achieves distances close to the solution
print("*** End optimization ***")
new_centers = []
for rbd in rigid_bodies:
    print("chain has", rbd.get_number_of_members(), \
        "atoms", "coordinates: ", rbd.get_coordinates())
    new_centers.append(rbd.get_coordinates())

d01 = IMP.algebra.get_distance(new_centers[0], new_centers[1])
d12 = IMP.algebra.get_distance(new_centers[1], new_centers[2])
d23 = IMP.algebra.get_distance(new_centers[2], new_centers[3])
d30 = IMP.algebra.get_distance(new_centers[3], new_centers[0])
print("Distances at the end of the optimization: d01 =", \
    d01, "d12 =", d12, "d23 =", d23, "d30 =", d30)



from __future__ import print_function
from IMP import ArgumentParser


__doc__ = "Plot score distributions of good-scoring models."


def parse_args():
    parser = ArgumentParser(
                description="Plot score distributions of good-scoring models")
    parser.add_argument("-show", action="store_true",
                        help="Interactively show the plot")
    parser.add_argument("score_file",
                        help="Score file generated by select_good")
    parser.add_argument("column",
                        help="PMI stat file column to plot (or 'all' to "
                             "plot all columns)")
    return parser.parse_args()


def plot_column(plt, scores, column, show):
    print(column, "|| Mean:", scores[column].mean(),
          "| SD:", scores[column].std())
    plt.hist(scores[column], bins=100)
    plt.xlabel(column)
    plt.ylabel('P')
    plt.title(column)
    if show:
        plt.show()
    else:
        plt.savefig(column+".png", dpi=300)
    return plt


def main():
    args = parse_args()

    import matplotlib
    matplotlib.use('Agg')
    import matplotlib.pyplot as plt
    import pandas

    scores = pandas.read_csv(args.score_file, sep=' ')

    if args.column == "all":
        print("Plotting all columns")
        exclude = frozenset(('Model_index', 'Replica_id', 'Frame_id',
                             'Run_id'))
        for k in scores.columns:
            if k not in exclude:
                p = plot_column(plt, scores, k, show=False)
                p.clf()
    elif args.column in scores.columns.values:
        plot_column(plt, scores, args.column, show=args.show)
    else:
        raise KeyError(args.column
              + " is not a valid score parameter. Use 'all' or one of: "
              + ", ".join(scores.columns.values))


if __name__ == '__main__':
    main()



## \example temperature_rem.py
# Temperature replica exchange

import IMP
import IMP.mpi
import IMP.core
import sys

IMP.setup_from_argv(sys.argv, "Temperature MPI example")

# min and max temperature
TEMPMIN_ = 1.0
TEMPMAX_ = 5.0

# initialize Replica Exchange class
rem = IMP.mpi.ReplicaExchange()
# get number of replicas
nproc = rem.get_number_of_replicas()
# create array of temperatures, in geometric progression
temp = rem.create_temperatures(TEMPMIN_, TEMPMAX_, nproc)
# get replica index
myindex = rem.get_my_index()
# set initial value of the parameter (temperature) to exchange
rem.set_my_parameter("temp", [temp[myindex]])

# create model
m = IMP.Model()

# add 2 particles
ps = []
for i in range(2):
    p = IMP.Particle(m)
    d = IMP.core.XYZ.setup_particle(p, IMP.algebra.Vector3D(0.0, 0.0, 0.0))
    d.set_coordinates_are_optimized(True)
    ps.append(p)

# add harmonic restraint to distance
h = IMP.core.Harmonic(5.0, 1.0)
ds = IMP.core.DistanceRestraint(m, h, ps[0], ps[1])
sf = IMP.core.RestraintsScoringFunction([ds])

# movers
movers = []
for p in ps:
    movers.append(IMP.core.BallMover(m, p, 0.5))
# serial mover
sm = IMP.core.SerialMover(movers)

# sampler
mc = IMP.core.MonteCarlo(m)
mc.set_scoring_function(sf)
mc.set_kt(temp[myindex])
mc.set_return_best(False)
mc.add_mover(sm)

# prepare output
log = open("log" + str(myindex), "w")

# start sampling loop
for istep in range(0, 100):
    # do optimization
    score = mc.optimize(100)

    # get my replica index and temperature
    myindex = rem.get_my_index()
    mytemp = rem.get_my_parameter("temp")[0]
    # score divided by kbt
    myscore = score / mytemp

    # printout
    log.write("%4d %2d %6.3f\n" % (istep, myindex, score))

    # get my friend index and temperature
    findex = rem.get_friend_index(istep)
    ftemp = rem.get_friend_parameter("temp", findex)[0]
    # score divided by kbt
    fscore = score / ftemp

    # try exchange
    flag = rem.do_exchange(myscore, fscore, findex)
    # if accepted, change temperature
    if (flag):
        mc.set_kt(ftemp)



## \example core/pair_restraint.py
# Restrain the distance between a pair of particles.
#

import IMP.example
import sys

IMP.setup_from_argv(sys.argv, "pair restraint")

(m, c) = IMP.example.create_model_and_particles()

uf = IMP.core.Harmonic(0, 1)
df = IMP.core.DistancePairScore(uf)
inds = c.get_indexes()
r = IMP.core.PairRestraint(m, df, (inds[0], inds[1]))
print(r.evaluate(False))



## \example atom/molecular_hierarchy.py
# In this example, we read a protein from a PDB file and set the center and radius of each residue to enclose the atoms in that residue.
#
# Then a second copy of the protein is loaded and they are both added to the same hierarchy
# to define a hypothetical assembly.
#

import IMP
import IMP.core
import IMP.atom
import sys

IMP.setup_from_argv(sys.argv, "molecular hierarchy")

m = IMP.Model()
mp0 = IMP.atom.read_pdb(IMP.atom.get_example_path('example_protein.pdb'), m)
# get the 16th residue of the first chain
hchain = IMP.atom.get_by_type(mp0, IMP.atom.CHAIN_TYPE)[0]
# decorate the chain particle with an IMP.atom.Chain decorator.
# unfortunately, our python wrapper does not handle converseions properly
# as a result you have to manually get the particle for that chain
chain = IMP.atom.Chain(hchain.get_particle())
r16 = IMP.atom.get_residue(chain, 16)
r16.show()

# get all the atoms
atoms = IMP.atom.get_by_type(mp0, IMP.atom.ATOM_TYPE)
# I didn't really have anything interesting to do with them...

# create a new version of the protein that is coarsened (one particle per
# residue)
smp0 = IMP.atom.create_simplified_along_backbone(chain, 1)

# we don't need mp0 any more
IMP.atom.destroy(mp0)

# load another copy
mp1 = IMP.atom.read_pdb(IMP.atom.get_example_path('example_protein.pdb'), m)

# create a graph from the hierarchy
mp1t = IMP.atom.get_hierarchy_tree(mp1)
# process the file with dot like
# dot -Tpdf hierarchy.dot > hierarchy.pdf
mp1t.show_graphviz(open("hierarchy.dot", "w"))
# try to display it graphically, assuming altgraph is installed
try:
    mp1t.show_with_altgraph()
except:
    pass

# make this one rigid
IMP.atom.create_rigid_body(mp1)

# create a hierarchy which contains the two proteins
p = IMP.Particle(m)
rmp = IMP.atom.Hierarchy.setup_particle(p)
rmp.add_child(smp0)
rmp.add_child(mp1)



def _show_histogram_1d(h, yscale, xscale, curves):
    import numpy as np
    import matplotlib.pyplot as plt
    import matplotlib.mlab as mlab
    fig = plt.figure()
    ax = fig.add_subplot(111)
    data = []
    countsgrid = h.get_counts()
    bins = [countsgrid.get_bounding_box(i).get_corner(0)[0]
            for i in countsgrid.get_all_indexes()]
    counts = h.get_counts().get_all_voxels()
    gbb = h.get_bounding_box()
    if yscale == 'linear':
        ax.bar(bins, counts, align='edge', width=bins[1] - bins[0])
    else:
        ax.plot(bins, counts, "bx-", linewidth=2)
    ax.set_xlim(gbb.get_corner(0)[0],
                gbb.get_corner(1)[0])
    ax.set_xscale(xscale)
    ax.set_yscale(yscale)
        # only scale based on histogram
    ax.set_autoscaley_on(False)
    for c in curves:
        ax.plot(bins, [c(x) for x in bins], "go-", linewidth=1)
    plt.show()


def _show_histogram_2d(h, yscale, xscale):
    import numpy as np
    import matplotlib.cm as cm
    import matplotlib.mlab as mlab
    import matplotlib.pyplot as plt
    cg = h.get_counts()
    steps = cg.get_unit_cell()
    x = np.arange(cg.get_bounding_box().get_corner(0)[0] + .5 * steps[0],
                  cg.get_bounding_box().get_corner(1)[0] - .5 * steps[0])
    y = np.arange(cg.get_bounding_box().get_corner(0)[1] + .5 * steps[1],
                  cg.get_bounding_box().get_corner(1)[1] - .5 * steps[1])
    X, Y = np.meshgrid(x, y)
    Z, junk = np.meshgrid(x, y)
    for i, xi in enumerate(x):
        for j, yj in enumerate(y):
            Z[i][j] = cg[cg.get_nearest_index(IMP.algebra.Vector2D(xi, yj))]
    im = plt.pcolor(X, Y, Z, cmap=cm.jet)
    plt.colorbar(im)
    plt.show()


def _get_min_dim(bb):
    md = 100000
    for i in range(0, bb.get_dimension()):
        l = bb.get_corner(1)[i] - bb.get_corner(0)[i]
        if l < md:
            md = l
    return md


def _show_histogram_3d(h, vmin, vmax):
    import IMP.display
    cg = h.get_counts()
    if not vmin or not vmax:
        minmax = h.get_minimum_and_maximum()
        if not vmin:
            vmin = minmax[0]
        if not vmax:
            vmax = minmax[1]
    print(vmin, vmax)
    if not IMP.display.PivyWriter:
        import sys
        sys.stderr.write("No pivy found\n")
        return
    w = IMP.display.PivyWriter()
    for idx in cg.get_all_indexes():
        v = cg[idx]
        # print v, vmin
        if v < vmin:
            continue
        coords = cg.get_center(idx)
        r = .25 * _get_min_dim(cg.get_bounding_box(idx))
        s = IMP.algebra.Sphere3D(coords, r)
        g = IMP.display.SphereGeometry(s, "histogram")
        scaled = (v - vmin) / (vmax - vmin)
        if scaled > 1.0:
            scaled = 1.0
        color = IMP.display.get_hot_color(scaled)
        g.set_color(color)
        w.add_geometry(g)
    w.show()


def show_histogram(h, yscale='linear', xscale='linear',
                   curves=[], vmin=None, vmax=None):
    if h.get_dimension() == 1:
        _show_histogram_1d(h, yscale, xscale, curves)
    elif h.get_dimension() == 2:
        _show_histogram_2d(h, yscale, xscale)
    elif h.get_dimension() == 3:
        _show_histogram_3d(h, vmin, vmax)
    else:
        raise ValueError(
            "Dimension " + str(h.get_dimension()) + "not supported")
